/* maxrdsc -- hst 04.05.2020 /27.02.2023 */

*! version 1.2.0  -- 27.02.2023 hst
*! - field rank corrected 
*! - in case of negative values, no absolute is taken anymore
*! version 1.1.1  -- 04.05.2020 hst
*! - If several ids have the same value, each id is counted once (minimum)
*! version 1.1.0  -- 30.01.2020 hst
*! - If several ids have the same value, each id is counted once (maximum)
*! version 1.0.1  -- 06.11.2019 hst  
*! - bug in stored results r(kmin) and r(kmax) corrected
*! version 1.0.0  -- 17.04.2019 hst  

capture program drop maxrdsc
capture program drop MinMax
program define maxrdsc, rclass
	
	syntax [varlist(default=none)] [if] [in] [, MAXimum(string) MINimum(string) NAME(string) REPLACE UPDATE NOITERate ABSolute]
	local osor : sortedby
	
	tokenize `varlist'		
	local id  = "`1'"
	local var = "`2'"	
	
	tempvar erlaubt touse hihi
	mark `touse' `if' `in'

	qui gen `hihi' = `var'

	di ""
	if "`maximum'"!="" {	
		if `maximum' < 5 {
			di as error "Sorry, maxrdsc requires at least 5 observations!"
			exit
		}
		if "`noiterate'"=="" {
			local k = 5
		}
		else if "`noiterate'"!="" {
			local k = `maximum'
		}
		local flag = 0 
		while `k' <= `maximum' & `flag'!=1 {
			cap drop `erlaubt'
			MinMax `id' `hihi' if `touse', max(`k') topcodename(`erlaubt') origname("`2'") `absolute' `noiterate'
			local flag = r(flagex)
			local k = `k' + 1
		}
		return scalar maxval = r(ext)
		local kk = `k'-1
		if `flag'==0 {
			di in red "D I S C L O S U R E problem: For variable `var' `kk' observations are not sufficient to determine maximum."
			exit			
		}
		return scalar kmax = r(rn)
		qui replace `hihi' = `erlaubt' if `hihi'!=`erlaubt' & `touse'
	}

	if "`minimum'"!="" {		
		if `minimum' < 5 {
			di as error "Sorry, maxrdsc requires at least 5 observations!"
			exit
		}
		if "`noiterate'"=="" {
			local k = 5
		}
		else if "`noiterate'"!="" {
			local k = `minimum'
		}
		local flag = 0 
		while `k' <= `minimum' & `flag'!=1 {
			cap drop `erlaubt'
			MinMax `id' `hihi' if `touse', min(`k') topcodename(`erlaubt') origname("`2'") `absolute' `noiterate'
			local flag = r(flagex)
			local k = `k' + 1
		}
		return scalar minval = r(ext)
		local kk = `kk' + `k'-1
		return scalar kmin = r(rn)		
		qui count if `var'!=.
		if `flag'==0 {
			di as errorin red "D I S C L O S U R E problem: For variable `var' `kk' observations are not sufficient to determine minimum."
			exit			
		}
		qui replace `hihi' = `erlaubt' if `hihi'!=`erlaubt' & `touse'
	}
	
	qui count if `var'!=.	/* Vergleich mit kk */
	if `kk'>r(N) {
		di in red "D I S C L O S U R E problem: For variable `var' observations for the calculation of the minimum and the maximum overlap."	
	}
	if "`update'"!="" {
		qui replace `var' = `hihi' 
	}
	else if "`name'"!="" & "`replace'"=="" {
		qui gen `name' = `hihi'
	}
	else if "`name'"!="" & "`replace'"!="" {
		qui replace `name' = `hihi'
	}
	
	if "`osor'" != "" {
		sort `osor'
	}	
end	


/* maximum, minimum und nex bezeichnen die Anzahl der Beobachtungen 
   rN                       bezeichnet die Anzahl der unterschiedlichen Banken  */

   
program define MinMax, rclass
*    version 15
	
	syntax [varlist(default=none)] [if] [in] [, MAXimum(string) MINimum(string) /*
	    */ TOPCodename(string) ORIGName(string) NOITERate ABSolute KRIT(int 5)]
		
	tokenize `varlist'		
	local id     = "`1'"
	local extvar = "`2'"	
	
	tempname touse 
	mark `touse' `if' `in'

	tempvar share hvsum tgid r tgr CR2 lfdnr eins mmax intvar tagidvar

	qui {

	generate `intvar' = `2'
	if "`absolute'"!= "" {
		replace `intvar' = abs(`intvar')
	}
	
	/* Sortiere absteigend im Falle des Maximums ... */
	if "`maximum'"!="" {
		egen `tagidvar' = tag(`intvar' `id')
		gsort -`touse' -`intvar' -`tagidvar'
		local extremum "maximum"
		local nex `maximum'
	}				
	/* ... und aufsteigend im Falle des Minimums. */
	else if  "`minimum'"!="" {
		egen `tagidvar' = tag(`intvar' `id')	
		gsort -`touse' `intvar' -`tagidvar'
		local extremum "minimum"
		local nex `minimum'				
	}

	generate `eins' = 1
	generate `lfdnr' = sum(`eins') if `touse' & `intvar'!=.
	/* Ermittle für eine bestimmte Variable den Anteil einer ID an allen IDs. */
	egen    `share' = total(`intvar') if `touse' & `intvar'!=. & `lfdnr'<=`nex', by(`id')
	egen    `hvsum' = total(`intvar') if `touse' & `intvar'!=. & `lfdnr'<=`nex'
	summarize `share' `hvsum'
	replace `share' = `share'/`hvsum'*100
	replace `share' = 0 if `hvsum'==0
	/* Ermittle daraus den Rang jeder ID, ... */
	egen    `tgid'  = tag(`id') if `touse' & `intvar'!=. & `lfdnr'<=`nex'
	
	generate `r' = sum(1)      if `share'>0	

	
	/* ... markiere jeden Rang einmal und ermittle den Anteil der beiden größten IDs. */
	egen byte `tgr' = tag(`r')  if `touse' & `intvar'!=. & `lfdnr'<=`nex'
	egen      `CR2' = total(`share') if `r'<=2 & `tgr'
	summarize `CR2'
	local cr = r(max)       						   /* concentration ratio */
	/* Ermittle das arithmetische Mittel der ersten "nex" IDs. */
	summarize `intvar' if `touse' & `intvar'!=. & `lfdnr'<=`nex' 
	local mex = r(mean)  /* Aus "nex" unterschiedlichen IDs gebildetes Mittel */

	count if `tgid'==1 
	local rN = r(N)
	}
	if (abs(`cr')<=85 | abs(`cr')>100)& `rN'>=`krit' {
		if "`topcodename'" != "" {
			qui generate `topcodename' = `extvar'
			qui replace  `topcodename' = `mex' if `lfdnr'<=`nex'
		}
		if "`noiterate'"=="" {
			di as text "No problems for `extremum' of `origname'. Average `extremum' based on  `rN'  distinct ids:  `mex'"	
		}
		else if "`noiterate'"!="" {
			di as text "No problems for `extremum' of `origname'. Average `extremum' based on  `nex'  observations:  `mex'"			
		}
	}
	return scalar flagex = 1
	return scalar ext    = `mex'
	else {
		if "`topcodename'" != "" {
			qui generate `topcodename' = `extvar'
			qui replace  `topcodename' =   .     if `lfdnr'<=`nex'
		}		
		return scalar flagex = 0	
	}
	return scalar rn = `rN'
end
