{smcl}

{title:Title}

{phang}
{bf:maxrdsc} {hline 2} Disclosure control and top coding for minimum, maximum and empirical cumulative distribution functions according to the 
rules of the Bundesbank RDSC

{marker syntax}{...}
{title:Syntax}

{p 8 17 2}
{cmdab:maxrdsc}
[{it:identifier variable}]
{ifin},
{opt minimum(string)}
{opt maximum(string)}
{opt name(varname)}
{opt replace}
{opt update}
{opt noiterate}


{marker description}{...}
{title:Description}

{pstd}
{it:Minimum} values and {it:maximum} values are single observations and must not be disclosed. If necessary, the RDSC 
allows to publish approximate values calculated as averages of a sufficient number of observations (top coding). The following requirements apply:
at least five distinct entities have to be covered and the share of the two largest ones must not exceed 85 percentages of the total (dominance criterion). 
{pstd}
If several identifiers show the same value these identifiers are flagged and sorted in such a way that the flagged observations are accounted for firstly.
{smcl}
{pstd}
The command {cmd:maxrdsc} followed by the name of an identifier, the name of a variable and at least the specification
of {it:mimimum} or {it:maximum} reports this approximation. {cmd:maxrdsc} increases the number of observations used 
for the approximation until the requirements are met or a user specified number of observations is reached. 
{smcl}
{pstd}
If you want to show a whole table you should use {cmd:nobsdes5}  with some preparatory steps.
{smcl}

{title:Options for use with {cmd:maxrdsc}}

{phang}
{cmd:min(}{it:string}{cmd:)} induces the approximated minimum to be calculated. The largest number of observations that is 
acceptable for the researcher must be specified in brackets.

{phang}
{cmd:max(}{it:string}{cmd:)} induces the approximated maximum to be calculated. The largest number of observations that is 
acceptable for the researcher must be specified in brackets.

{phang}
{cmd:name(}{it:varname}{cmd:)} supplies the name of a new variable which contains the top coded observations of the variable under 
scrutiny. If the requirements are not met, observations are replaced by missing values, whereby their number is given by {cmd:min(}{it:string}{cmd:)} or {cmd:max(}{it:string}{cmd:)}.

{phang}
{cmd:replace} overwrites the variable specified by {bf:name(}{it:varname}{cmd:)}.

{phang}
{cmd:update} updates the existing variable. 

{phang}
{cmd:noiterate} {cmd:maxrdsc} calculates the approximation using the number of observations specified in brackets.


{marker examples}{...}
{title:Examples}

1) Display minimum and maximum for several variables.

foreach var in turnover imports value_added {
	maxrdsc num `var', min(12) max(9)
}

No problems for turnover. Average maximum based on 5 distinct ids: 7.464013
No problems for turnover. Average minimum based on 5 distinct ids: 0

No problems for imports. Average maximum based on 5 distinct ids: 4.071133
No problems for imports. Average minimum based on 5 distinct ids: 0

No problems for value_added. Average maximum based on 5 distinct ids: 6.724842
No problems for value_added. Average minimum based on 6 distinct ids: 2.721722


2) Create top-coded variables for a graph of empirical cumulative distribution functions.  
The loop over countries may replace values in `var'_tp if you do not use a different 
variable for the countries. This could make reidentification very easy and result in 
different outcomes for the total maximum depending on whether it is calculated before 
or after the loop over the countries.

foreach var in turnover imports value_added {
	maxrdsc num `var', min(12) max(9) name(`var'_tp) replace 
}

foreach var in turnover imports value_added {
	qui gen `var'_land_tp = `var'
	foreach land of numlist 400 1 6 39 {
		maxrdsc num `var'_land_tp if land==`land', min(12) max(9) update 
	}
}

/* Remark:
Top-coding produces ties so you should specify 'equal' with the cumul command.
*/
	
cumul turnover_tp                  , gen (cum_1) equal 		/* all countries */
cumul turnover_land_tp if land==400, gen (cum_2) equal		/* USA           */
cumul turnover_land_tp if land==  1, gen (cum_3) equal		/* France        */
cumul turnover_land_tp if land==  6, gen (cum_4) equal		/* UK            */
cumul turnover_land_tp if land== 39, gen (cum_5) equal		/* Switzerland   */


{marker results}{...}
{title:Stored results}

{pstd}
{cmd:maxrdsc} stores the following in {cmd:r()}:

{synoptset 15 tabbed}{...}
{p2col 5 15 19 2: Scalars}{p_end}
{synopt:{cmd:r(minval)}}value of approximated minimum{p_end}
{synopt:{cmd:r(maxval)}}value of approximated maximum{p_end}
{synopt:{cmd:r(kmin)}}number of distinct entities for approximating the minimum{p_end}
{synopt:{cmd:r(kmax)}}number of distinct entities for approximating the maximum{p_end}
{p2colreset}{...}


{marker Author}{...}
{title:Author}
		Harald Stahl
		Harald.Stahl@bundesbank.de
        
{marker Alsosee}{...}
{title:Also see}

		{help kdensity}

Further examples: 
R:\Zentrale\ZB-S\Daten_FDSZ_Gastforscher\1_gafo\examples_rdsc\maxrdscxmpl.do

