/* maxrdscxmpl.do -- hst 04.03.2019 / 03.06.2019 */

clear

/* *********************************************************************** */
/*                                                                         */
/*                       Create artificial data for                        */
/*                                                                         */
/*                   M A X I M U M   and   M I N I M U M                   */
/*                                                                         */
/*    11 IDs:             1-  11                                           */
/*    20 years:        2000-2019                                           */
/*     3 countries:  17, 39, 400                                           */
/*                                                                         */
/* *********************************************************************** */   

set obs 601
gen land = int((_n-1)/200)+1
gen hv   = _n-200*(land-1)
gen id   = int((hv-1)/20)+1
gen year = 1999+hv-20*(id-1)
drop hv
replace id = 11 if _n==601 /* id has to be ignored, since x==. (see below) */

replace land =  17 if land==1
replace land =  39 if land==2
replace land = 400 if land==3

set seed 4869382
gen x = runiform()

* create missings
replace x = . if year==2002 & id > 2
replace x = . if id==11

* create negative values
gen y = rnormal(.6)

cls

/* *********************************************************************** */   
/*                                                                         */
/*                              M A X R D S C                              */
/*                                                                         */
/*     M A X I M U M   and   M I N I M U M   are single observations.      */
/*     Thus, they must not be shown. If necessary, the RDSC allows to      */
/*     publish approximate values calculated as averages of a sufficient   */
/*     number of observations (top coding). The following requirements     */
/*     apply: at least five distinct entities have to be covered and the   */
/*     share of the two largest ones must not exceed 85 percentages of the */
/*     total (dominance criterion). The dominance criterion is the reason  */
/*     why in the next example six observations are needed.                */
/*                                                                         */
/* *********************************************************************** */

maxrdsc id x, min(12) 
* For comparison reasons:
sort x
list id x in 1/6       /* you need 6 observations, since id 1 occurs twice */
sum x in 1/6

/* *********************************************************************** */   
/*                                                                         */
/*     In the next example at most 12 observations are accepted to         */
/*     approximate the maximum. It is possible to specify minimum and      */
/*     maximum at the same time.                                           */
/*                                                                         */
/* *********************************************************************** */

maxrdsc id x, max(12) 
list id year land x in 567/576    /* largest 10 observations */
sum x in 567/576

maxrdsc id x, min(9) max(12)
maxrdsc id x, min(12) max(9) 

foreach i in 17 400 {
	maxrdsc id x if land==`i', min(12) max(12) 
	display
}

/* *********************************************************************** */   
/*                                                                         */
/*                               N O I T E R A T E                         */
/*                                                                         */
/*     The researcher can require a specific number of observations        */
/*     without iteration.                                                  */
/*                                                                         */
/* *********************************************************************** */

maxrdsc id x, min(10) max(10) noiterate
sum x in 567/576
sum x in 1/10

/* *********************************************************************** */
/*                                                                         */
/*                             A B S O L U T E                             */
/*                                                                         */
/*     In some cases the researcher may prefer maximum or minimum of the   */
/*     absolute values.                                                    */
/*                                                                         */
/* *********************************************************************** */

maxrdsc id y, min(12) max(12) absolute

*  You need six observations for the minimum so that the 
*  dominance criterion is fullfilled.

/* *********************************************************************** */
/*                                                                         */
/*                         R E P L A C I N G   V A L U E S                 */
/*                                                                         */
/*     E. g. for making a graph of an empirical cumulative distribution    */
/*     function one has to replace existing values. The researcher can     */
/*     provide his own variable or use the option  N A M E ()  and         */
/*     specify only the name of a new variable.                            */
/*                                                                         */
/* *********************************************************************** */

gen y_user = y
maxrdsc id y_user, min(12) max(9) update  

* or

maxrdsc id y, min(12) max(9) name(y_tc)   /* tc for topcode */
cumul y_tc, gen(y_tc_cum) equal
line y_tc_cum y_tc, sort 

* Top-coding produces ties so you should specify 'equal' with the cumul command.

/* *********************************************************************** */
/*                                                                         */
/*     In case of entire tables the researcher has to use two variables,   */
/*     one for the total and another for the breakdown. Otherwise some of  */
/*     the largest values regarding the total may be replaced by averages  */
/*     of the following samples.                                           */
/*                                                                         */
/* *********************************************************************** */

maxrdsc id x, min(12) max(12) name(x_tc) 

gen x_user = x
foreach i in 17 400 {
	maxrdsc id x_user if land==`i', min(15) max(15) update
}
sort x
list id land year x x_tc x_user if x_tc!=x & x!=.
sort land x
list id land year x x_tc x_user if x_user!=x & x!=., sepby(land)

/* If you want to show results outside the RDSC you have to create a table!
   The following shows a possibility using the stored results 

   r(minval)      value of approximated minimum
   r(maxval)      value of approximated maximum
   r(kmin)        number of distinct entities for approximating the minimum
   r(kmax)        number of distinct entities for approximating the maximum  */

foreach i in 17 400 {
	display "country:  `i'"
	qui maxrdsc id x if land==`i', min(12) max(12)
	local min_`i' = r(minval)
	local max_`i' = r(maxval)
	qui sum x  if land==`i'
	local n_`i'    = r(N)
	local mean_`i' = r(mean)
	local sd_`i'   = r(sd)
}   
   
foreach i in 17 400 {
    di %30s "`i' "  %10.0f `n_`i'' %18.4f `mean_`i''   %18.4f `sd_`i''       %18.4f `min_`i''      %18.4f `max_`i''
}   
   
/* *********************************************************************** */   
/*                                                                         */
/*                        K e r n e l  d e n s i t y                       */
/*                                                                         */
/*     If the researcher's interest lies in the probability density        */
/*     function he may use kernel density estimates.                       */
/*                                                                         */
/* *********************************************************************** */

kdensity y
