/* nobsdes5.ado -- hst 06.03.2020 / 30.06.2020 / 28.01.2021 / 06.01.2022 / 17.02.2022 
                     / 29.04.2022 / 02.06.2022 / 11.07.2022 / 22.08.2022 / 28.12.2022 
		     / 27.02.2023 / 10.08.2023 / 25.08.2023 / 06.09.2023 / 15.09.2023 */

/* nobsdes5.ado  ermittelt fuer mehrere Variablen die Anzahl der unterschiedlichen 
   IDs mit validen Beobachtungen. Benoetigt wird der Name der ID und der 
   interessierenden Variablen (in dieser Reihenfolge).  Die Optionen 'histogram'
   und 'pctile' werden in eine 'by'-Option transformiert. Da in diesen Fällen 
   für jede Variable unterschiedliche 'by'-Optionen resultieren, muss jede 
   Variable einzeln untersucht werden. Es können nicht mehrere Variablen mit 
   einer einheitlichen 'by'-Option verdichtet werden. */

*! version 2.2.1  09.11.2023   
*! - output of option `pctile' slightly reworked
*! version 2.2.0  20.09.2023   
*! - option `pctile' reworked so that quantiles are treated as deterministic 
*!   categories i.e. an ordinary table with option by(bin). In case of negative 
*!   values no absolute is taken anymore to evaluate the dominance criterion. 
*!   Now, the critical range is 0.85<x<1. Ratios > 1 are permitted.
*! version 2.1.10  14.09.2023   
*! - option `histogram' reworked so that bins are treated as deterministic 
*!   categories i.e. an ordinary table with option by(bin).   
*! version 2.1.9  12.09.2023
*! - using 'generate' and miss(0) in the standard case ignored the miss(0) option.    
*! version 2.1.8  06.09.2023
*! - option `by' did not detect a disclosure problem if the concentration ratio 
*!   in at least one cell was larger than 100 percent.
*! - option `byest' not valid anymore
*! version 2.1.7  25.08.2023
*! - option `pwcorr' for pairwaise correlation included. Not that we think it is
*!   really needed, but researchers asked for it 
*! version 2.1.6  10.08.2023
*! - when running the pctile option an error could occur. The accents for local  
*!   variable `border' were lost.
*! version 2.1.5  14.07.2023
*! - the option generate did not work when used with the pctile option
*! - the code for percentile option was simplified and corrected. Now, for the 
*!   smallest percentile always an observation is counted. Previously, this 
*!   could have been a missing. 
*! - for the calculation of the concentration ratio the egen option "rank, field" 
*!   was used causing wrong ranks if the largest values were equal. Ranks are 
*!   now calculated explicitly.
*! - in case of negative values no absolute is taken anymore to evaluate the 
*!   dominance criterion. Now, the critical range is 0.85<x<1. Ratios > 1 are 
*!   permitted.
/* *! - in the case of sufficient distinct entities but a violation of the dominance
*!   criterion together with the by option and the pctile option 
*!   "D I S C L O S U R E problem: " was displayed but no problematic output. */
*! version 2.1.4  07.09.2022
*! - no warning anymore in case of a known identifier without further variables   
*! version 2.1.3  23.08.2022
*! - requiers STATA 16 or greater
*! - code for STATA 15 deleted 
*! - comments added 
*! - several checks on input added e.g. whether percentiles are ordered 
*! - nobsdes5 exits if mistakes are too severe
*! - some local variable names replaced
*! - smcl used for italics and bold face   
*! version 2.1.2  01.07.2022
*! - if there is a missing value in `by' a warning occurs
*! - previously, if a variable label was left blank [|] was displayed in the ouput
*! - in case of for example nobsdes5 id x1 x2 x3, miss(0) without further options
*!   the output is less repetitive than before
*! version 2.1.1  02.06.2022
*! - previous correction induced a new error with the effect that not at least
*!   five distinct entities were requiered but two, as with nobsdes. This has
*!   been fixed.
*! - the calculations for pctile were incorrect. Instead of the n-th observation
*!   the n+1-th observation was taken as percentile.
*! - The message 'Zero values for var found' is not displayed anymore with the
*!   pctile option.
*! - using the option 'generate' left a variable called 'rdscfrnobs' behind.
*!   This variable is now dropped after execution.
*! version 2.1.0  29.04.2022
*! - error with varlist if 'crosstabs': within the loop all rows with missings 
*!   (and zeros) were deleted. Thus the data frame became smaller and smaller.
*!   A conservative error because the number of observations became smaller and
*!   smaller.
*! version 2.0.3  17.02.2022
*! - error message in case of dummy variable in varlist
*! version 2.0.2  14.01.2022
*! - error with `by' in output corrected `tg_id_by'==1 instead of `by'!=`by'[_n-1]
*! version 2.0.1  10.01.2022
*! - error with `by' in output corrected `tg_id_by'==1 instead of `by'!=`by'[_n-1]
*! version 2.0.0  07.01.2022
*! - allows in most cases varlist, e.g.  nobsdes5 id x xpc, by(country)
*!   or nobsdes5 id x xpc, histogram(25 10)
*! - runs under STATA 15 as well as under STATA 16 and STATA 17
*! - part of code moved to subroutines 
*! version 1.0.2  17.11.2021
*! - in case of a break, previously the variables nobs, CR2, pctile, and value
*! - remained in the dataset. Now, these are dropped
*! version 1.0.1  13.06.2021
*! - variable labels in output added
*! version 1.0.0  28.01.2021
*! - option `histogram' is now available. Syntax:


capture program drop nobsdes5
program define nobsdes5, rclass
	local vs = c(stata_version)
	local vu = c(userversion)
	version `vs'

	if `vs' < 16 {
		display as error "STATA 16 or higher needed"
		exit
	}
	
	syntax [anything(name=clist)] [if] [in] [, BY(varlist) BYEST(varlist)  /*
	    */ BYREGIO(varlist) BYDUPL(varlist) MISS(string) GENerate Name(string)  /*
		*/ REPLACE NOTABle PCTILE(string) HISTogram(string) PWCORR ENDmark(string) SHOW]
		
	local osor : sortedby
	local _minobs 5

	local anzvar : word count `clist'		
	tokenize  `clist'
	
	tempname  touse 
	mark     `touse' `if' `in'

	/* --------------------------   Command line   -------------------------- */	
	
	local commandline "nobsdes5 `clist'"
	if "`if'"!="" {
		local commandline "`commandline' `if'"
	}
	if "`in'"!="" {
		local commandline "`commandline' `in'"
	}
	if "`by'"!="" | "`byest'"!="" | "`byregio'"!="" | "`bydupl'"!="" | "`miss'"!="" | "`gen'"!="" | "`name'"!="" | "`replace'"!="" | "`notab'"!="" | "`pctile'"!="" | "`hist'"!="" {
		local options ","
	}
	if "`by'"!="" {
		local options "`options' by(`by')"
	}
	if "`byest'"!="" {
		local options "`options' byest(`byest')"
		display as error "Option `byest' not valid anymore"
	}
	if "`byregio'"!="" {
		local options "`options' byregio(`byregio')"
	}
	if "`pctile'"!="" {
		local options "`options' pctile(`pctile')"
	}
	if "`hist'"!="" {
		local options "`options' hist(`hist')"
	}	
	if "`miss'"!="" {
		local options "`options' miss(`miss')"
	}
	if "`bydupl'"!="" {
		local options "`options' bydupl(`bydupl')"
	}
	if "`gen'"!="" {
		local options "`options' gen(`gen')"
	}
	if "`name'"!="" {
		local options "`options' name(`name')"
	}
	if "`notab'"!="" {
		local options "`options' notab)"
	}
	if "`replace'"!="" {
		local options "`options' replace(`replace')"
	}

    *noisily display "`commandline'`options' [version 2.5.1]"
	
	/* ---------------------------   Check input   -------------------------- */
	**# Check input
	
	if `anzvar'==1 & "`1'"!="BAID" & "`1'"!="BAID_DOM" & "`1'"!="BAID_MOF" & "`1'"!="BAID_FIL" & "`1'"!="BAID_CON" & "`1'"!="SYSNR" & "`1'"!="SYSNR_DOM" & "`1'"!="SYSNR_MOF" & "`1'"!="SYSNR_FIL" & "`1'"!="SYSNR_CON" & "`1'"!="AWMUS_CD" & "`1'"!="AWMUSID" & "`1'"!="BBK_HM_CO_ID1" {
		display as error "Warning" as text " Is " as smcl "{it:`1'}" _asis " an identifier?"
	}
	
	/* Is id string or numeric ? */
	capture confirm string variable `1'
	if _rc==0 {
		local   id "`1'"
	    quietly replace `touse' = 0 if `id'==""
		quietly count if `id'==""
		local Nmiss = r(N)
	}
	else if _rc!=0 {
		local   id "`1'"
	    quietly replace `touse' = 0 if `id'==.
		quietly count if `id'==.
		local Nmiss = r(N)		
	}
	if `Nmiss'!=0 {
		display as error "Warning" as text " There are missing values in identifier " as smcl "{it:`1'}" _asis "."	
	}
	
	
	/* Is identifier or variable name missing ? */
	if ("`histogram'"!="" | "`pctile'"!="") & `anzvar'<2 {
		display as error "You need an identifier and at least one further variable."
		exit
	}
	
	
	/* Are there missing values in `by' (numeric string) ? */
	if "`by'"!="" {	
		foreach var in `by' {
			capture confirm numeric variable `var'
			if !_rc {
				capture assert `var'!=.
				if _rc!=0 {
					display as error "Warning" as text " Missing values for variable " as smcl "{it:`var'}" _asis " found -> Do you want to restrict " as smcl "{bf:if `var'!=.}" _asis " ?"
				}
			}
			else {
				capture assert `var'!=""
				if _rc!=0 {
					display as error "Warning" as text " Missing values for variable " as smcl "{it:`var'}" _asis " found -> Do you want to restrict " as smcl "{bf:if `var'!=`""'}" _asis " ?"

				}
			}
		}
	}

	
	/* Setting kritical value for regional data */
	if "`byregio'" != "" {
		local by = "`byregio'"	
		local _minobs 20 
	}
	
	
	/* Does (dupl) show up in clist? Create varlist without (dupl) */
	if "`bydupl'"!="" {
		local varlist ""
		local i 1
		while `i' <= `anzvar' {
			local var : word `i' of `clist'	
			if "`var'" != "(dupl)" {
				local varlist `varlist' `var'
				local i = `i' + 1							
			}
			else {
				local i = `i' + 1  /* 2 */
			}
		}
	}
	else {
		local varlist `clist'
		local anzvar : word count `varlist'		
		tokenize  `varlist'
	}


	/* New variable `name' which contains the number of distinct values of the 
	   varlist by levels defined by the categorical variable supplied in the 
	   by option. */
	if "`name'" == "" {
		tempvar   name 
	}	
	if "`replace'"!="" {
		capture drop `name'
	}	

    
	/* Is miss option requiered? Umsetzung der miss-Option erfolgt später */
	if `anzvar'>=2 {
		tokenize `varlist', parse(" ")
		macro shift
		local noidlist `*' 		
		foreach var in `noidlist' {
			capture assert `var'==0|`var'==1|`var'==.     /* Is dummy? */
			if _rc!=0 {
				capture assert `var'!=0
				if _rc!=0  & "`miss'"=="" & "`pctile'"=="" {		
					display as error "Warning" as text " Zero values for variable " as smcl "{it:`var'}" as text " found -> Should you use option " as smcl "{bf:miss(0)}" _asis " ?"
				}
			}
			else {
				display as smcl "{error}{it:`var'}" _asis " is a " as error "0-1 dummy variable. " as text "Dummy and categorial variables " as error "must not " as text "be part of varlist but " as error "must " as text "be included in by() -> " as smcl "{bf:by(`var')}" _asis
				exit
			}
		}
	}

	/* Generate extvar to manipulate extvar and labels */
	if `anzvar'==1 {
		local   extvar = 1
		local   kritcr = 0
	}
	
	
	if "`histogram'"!="" & `touse' {
		if "`show'"=="" {
			local notable = "notable"
		}
	}

	
	/* Inquiery, whether percentiles outside the admissible range are not sorted */
	if "`pctile'"!="" {	
		tokenize "`pctile'"
		local pctilelist `*'		
		local pctold = 0
		local hv = 0
		foreach pct in `pctilelist' {
			if "`pct'"!="|" {
				if lower(substr("`pct'",1,1))=="p" {
					noi di as error "Please specify only figures, e.g. 5 but not p5"	
					exit
				}
				if `pct'<=0 | `pct'>100 {
					noi di as error "Percentile (`pct') has to be greater than 0 and smaller than 100"
					exit				
				}			
				if `pct'<=`pctold' & `hv'==0 {
					noisily {
						display as error "Percentiles not sorted:" as text " `pctold' is larger than `pct'"
					}
					exit
				}
				local pctold = `pct'
				local hv = 0
			}
			else if "`pct'"=="|" {
			    local hv = 1	
			}
		}	
	}
	
	/* Here, `name' is dummy variable for non missings tag() */	
	qui egen byte `name' = tag(`id' `by') if `touse' 

	if "`generate'"!="" & `anzvar'>=2 {
		foreach var in `noidlist' {	
			if "`replace'"!="" {
				cap drop tg_`id'_`var'
				cap drop cr2_`id'_`var'				
			}
			qui generate byte  tg_`id'_`var' = `name'
			qui replace        tg_`id'_`var' = 0       if  `var'==.
		}
	}
	else if "`generate'"!="" & `anzvar'==1 {
		if "`replace'"!="" {
			cap drop tg_`id'
		}	
		qui generate byte tg_`id' = `name'	
	}

	if "`show'"!="" {
		noi di "Input check passed"
	}

    /* ---------------------------   Evaluations   -------------------------- */
	**# Evaluations
	
	/* Calculate concentration ratio for the two largest entities CR2 */
	/* was wird weiterverwendet? */

	**# histogram
	if "`histogram'"!="" {
		local anzvar : word count `varlist'
		local anzhis : word count `histogram'
		qui forvalues i = 2/`anzvar' {
			local extvar : word `i' of `varlist'	
			if `anzhis'==1 {
				local hist = `histogram'						
			}
			else {
				local j = `i'-1				
				local hist : word `j' of `histogram'	
			}
			preserve
			if "`miss'"!="" {
				replace `extvar' = . if `extvar'==`miss'
			}
			drop if   `touse'==0
			tempvar    binvar        /* binvar   = bins */
			summarize `extvar'	
			generate  `binvar'   = 1 + int(`hist'*(`extvar'-r(min))/(r(max)-r(min) ))  if `extvar'!=.
			replace   `binvar'   = `binvar' - 1 if `binvar'==`hist'+1
			local labextvar : variable label `extvar'			
			collapse (sum) `extvar', by(`id' `by' `binvar')
			tempvar   id_tot tot share name r CR2 crdum hv

			/* -----------   Calculation of concentration ratio   ----------- */			
			egen      `id_tot'   = total(`extvar'),				by(`id')           
			egen      `tot'      = total(`extvar'),				by(`by' `binvar') 
			generate  `share'    = `extvar'/`tot'*100                 
			replace   `share'    = 0 if `tot'==0	
			egen      `name'     = total(`extvar'!=0),			by(`by' `binvar') 	
			gsort `by' `binvar' -`share'
			if "`by'"=="" {
				by `binvar' : generate `r'   = sum(1)           if `name'>0
				egen                   `CR2' = total(`share')	if `r'<=2, by(`binvar')
			}
			else {
				by `by': generate `r' = sum(1)           if `name'>0
				egen     `CR2'		  = total(`share')	 if `r'<=2, by(`by' `binvar')
			}			
			* max CR2 may be larger than 100 percent, therefore create crdum
			* if there are cells with abs(`CR2') > 85 & abs(`CR2') <= 100, then
			* kritcr refers to thsi range; otherwise it refers to the largest
			* CR2 value
			generate  `crdum'    = abs(`CR2') > 85 & abs(`CR2') <= 100
			summarize `crdum'
			if r(max)==1 {
			    summarize `CR2' if `crdum' == 1				
			    local kritcr   = abs(r(mean))				
			}
			else {
			    summarize `CR2'				
				local kritcr   = abs(r(max))
			}
			format %8.0f `CR2'						
			label variable `CR2'       "CR2"
			
			/* -------------------------------------------------------------- */			
			label variable `name'      "nobs"
			qui generate `hv' = `name'
			replace   `name'     = . if `r'!=1  
*			drop if   `r'!=1
			summarize `name'
			local kritnobs = r(min)
			if (r(min)>0 & r(min)<`_minobs') {
	 	        local rmin = "too small"
	        }
	        else {
		        local rmin = r(min)
	        }
			
			if "`show'"!="" {
				noi di "Step 3 : histogram calculations for `extvar' passed"
				noi table `by' `binvar',  statistic(mean `name' `CR2') nototal nformat("%8.0f")										
			}
			
			/* ------   Report (message) in case of histogram option   ------ */

			noi {
			display ""
			display as text "Output control for variable " as smcl "{it:`extvar' [`labextvar']}"	_asis
			if (`kritnobs'<`_minobs' & `kritnobs'>0)|(`kritcr'>85 & `kritcr'<=100) {	
				display as error "D I S C L O S U R E problem: "
			}					
			if `kritcr'>85 & `kritcr'<=100 {
				display as text "Share of largest two IDs > 85% and <= 100%" 
			}				
			else if `kritcr'<=85 | `kritcr'>100 {
				display as text "No problem with dominance " 
			}
			if (`kritnobs'>=`_minobs') & (`kritcr'<=85 | `kritcr'>100) {
				display as text "Number of distinct IDs (" as smcl "{it:`id'}" _asis ") for the smallest of `hist' bins of variable " as smcl "{it:`extvar'}" _asis " : `rmin'"
				display ""	
			}
			
			if (`kritnobs'<`_minobs') | (`kritcr'>85 & `kritcr'<=100) {
				display as text "Number of distinct values for identifier " as smcl "{it:`id'}" _asis " for each of `hist' bins of variable " as smcl "{it:`extvar'}" _asis " :"	
				nobreak {
					sort   `binvar'
					rename `hv' nobs
					rename `binvar' bin
					rename  `CR2' CR2
					tempvar tg_id_by
					egen   `tg_id_by' = tag(bin)       if nobs!=. & CR2!=.
					list    bin nobs CR2               if `tg_id_by'==1, noobs sep(0)
					rename  CR2 `CR2'
					rename  nobs `hv'
					rename  bin `binvar'
				}
				display ""
			}
			}
			restore
		}
		if "`show'"!="" {
			noi di "Step 4 histogram completed"
		}
	}
	

	**# pctile
	if "`pctile'"!="" {
		capture frame drop rdscfrnobs
		if _rc==0 {
			display as error "Frame rdscfrnobs found and dropped. Probably, nobsdes5 did not terminate correctly last time."
		} 
		* The variables pgr for the categories defined by the percentiles have to be created outside for linking with default frame
		local anzvar : word count `clist'
		local i 2
		local j 2
		* data frame allows list of several variables
		* mypctile allows only one variable, loop
		qui while `j' <= `anzvar' {
			local extvar : word `j' of `clist'
			/* anders hochzählen extvar_`i' : word `j' of `clist', Dopplung */
			if "`bydupl'"=="" {	
				tempvar extvar_nvalid
				if "`miss'"!="" {
					generate `extvar_nvalid' = (`extvar'!=. & `extvar'!=`miss')
				}
				else if "`miss'"=="" {
				    generate `extvar_nvalid' = (`extvar'!=.)
				}
			}
			else if "`bydupl'"!="" {	
				tempvar extvar_nvalid ndupl		
				if "`extvar'" != "(dupl)" {
					generate `extvar_nvalid' = (`extvar'!=.)	
				}
				else if "`extvar'" == "(dupl)" {		
					local j = `j' + 1				
					local extvar : word `j' of `clist'	
					generate `extvar_nvalid' = (`extvar'!=.)						
					egen    `ndupl'      = total(`extvar_nvalid'), by(`bydupl')
					if "`show'"!="" {
						noi di "Step 3: `extvar'    ndupl"
						noi table `bydupl', statistic(mean `ndupl') nototals
					}
				}
			}				
			else {
				tempvar  ndupl
				generate `ndupl' = 1 
			}
			
			tokenize "`pctile'", parse("|")	
			local pctile1 `1'
			if "`2'"=="|" {
				macro shift 2
				local pctile `*'
			}
			capture drop `extvar'_pgr
			capture drop `extvar'_pgrstr			
			/* Benennung der Quantile */		
			generate `extvar'_pgr 	  = 0   if `extvar'!=.  
			generate `extvar'_pgrstr  = 0	if `extvar'!=.  
			tempvar  extvar1 
			generate `extvar1' = `extvar'
			
            * mypctile calculates the empirical values for `extvar'_pgr and 
			* `extvar'_pgrstr [weight=`ndupl']
			* echte missings (.) werden nicht betrachtet, Nullen schon
	        mypctile `id' `extvar1' `extvar'_pgr `extvar'_pgrstr   if `extvar1'!=. & `touse', pctile("`pctile1'") by(`by') bydupl(`bydupl') 
			local pstring = r(pstring)
			local labextvar : variable label `extvar'		
			capture frame drop `rdscfrnobs'		
		    tempname rdscfrnobs
			frame put `id' `extvar1' `extvar' `extvar_nvalid' `extvar'_pgr `extvar'_pgrstr `by' `bydupl' `touse', into(`rdscfrnobs')	
		    frame change `rdscfrnobs'
			keep if `touse'
			collapse (sum) `extvar1' `extvar' `extvar_nvalid' (mean) `extvar'_pgrstr  if `extvar1'!=., by(`id' `by' `extvar'_pgr) 
			sort `extvar'_pgr, stable
			tempvar    id_tot minshare tot share name r CR2
			egen      `id_tot'   = total(`extvar1')               if `extvar_nvalid'!=0, by(`id')  
			* kleinster Anteil über alle Einheiten je Zelle
			egen      `minshare' = min(`extvar1'/`id_tot'*100)    if `extvar_nvalid'!=0, by(`by' `extvar'_pgr) 
			egen      `tot'      = total(`extvar1')               if `extvar_nvalid'!=0, by(`by' `extvar'_pgr) 
			generate  `share'    = `extvar1'/`tot'*100            if `extvar_nvalid'!=0
			replace   `share'    = 0                              if `extvar_nvalid'==0 & `tot'==0	              
			egen      `name'     = total(`extvar_nvalid'!=0),	  by(`by' `extvar'_pgr) 	
			gsort     `by' `extvar'_pgr -`share'	
 		    local BY `"by `by' `extvar'_pgr:"'
  	        `BY' generate `r' = sum(1)      if `name'>0 
			egen      `CR2'		 = total(`share')	 if `r'<=2,   by(`by' `extvar'_pgr)
			format %8.0f   `CR2'								
			label variable `name'      "nobs"
			label variable `CR2'       "CR2"
			sort `by' `extvar'_pgr `id' 
			if "`show'"!="" {
				noi di "Step 3 (pctile): `extvar1'"
				noi display "       by   extvar       tot     id       pgr     pgrstr   count        CR2      share "					
				noi list `by' `extvar1' `tot' `id'  `extvar'_pgr  `extvar'_pgrstr  `name' `CR2' `share', sepby(`by')		
				noi table `extvar'_pgr, statistic(mean `name' `CR2') nototal nformat("%8.0f")
			}
			
			/* -------------------------------------------------------------- */
			
			/* Zwischen kleinstem und größtem Quantil (`binvar'>1 & `pcmax'==0), 
			Fallzahl zu gering (`totname'> 0 & `totname'< `_minobs') und 
			Dominanzkriterium verletzt (`minshare'>85) */
			noi mess_pctile `id' `extvar1' `CR2' `name' `extvar'_pgr `extvar'_pgrstr `extvar', `notable' minobs(`_minobs') labelby(`labextvar') by(`by') pstring("`pstring'") pctile("`pctile1'")

		    /* Link: collapse (sum) `extvar' (mean) `pgrstr', by(`id' `by' `pgr')		
		       determines the variables necessary for linking: `id' `by' `pgr' */
			if "`generate'" != "" {
				tempvar hv
				egen    `hv'               = mean(`CR2'),  by(`extvar'_pgr `by') 
				generate cr2_`id'_`extvar' = `hv'
	            capture frame change default
				capture drop rdscfrnobs	
				capture drop tg_`id'_`extvar'	
			    if "`by'"=="" {
					if "`miss'"=="" {
					    egen byte  tg_`id'_`extvar' = tag(`id' `extvar'_pgr)   if `extvar1'!=. & `touse'
					}
					else if "`miss'"!="" {
					    egen byte  tg_`id'_`extvar' = tag(`id' `extvar'_pgr)   if `extvar1'!=.  & `touse' & `extvar1'!=`miss'
					}					
		    	    frlink m:1 `id' `extvar'_pgr, frame(`rdscfrnobs') 
			    }
			    else if "`by'"!="" {
					if "`miss'"=="" {
					    egen byte  tg_`id'_`extvar' = tag(`id' `extvar'_pgr `by')   if `extvar'!=. & `touse'	
					}
					else if "`miss'"!="" {
					    egen byte  tg_`id'_`extvar' = tag(`id' `extvar'_pgr `by')   if `extvar'!=. & `touse'	& `extvar'!=`miss'
					}					
			        frlink m:1 `id' `by' `extvar'_pgr, frame(`rdscfrnobs') 					
			    }
			    frget cr2_`id'_`extvar', from(`rdscfrnobs')
			    quietly replace cr2_`id'_`extvar' = .    if tg_`id'_`extvar'==0
 		        capture frame change `rdscfrnobs'
 		    }  
			frame change default
			frame drop `rdscfrnobs'
			* drop `extvar'_pgr `extvar'_pgrstr 
			local j = `j' + 1	

		}		
		if "`show'"!="" {
			noi di "Step 4 (pctile)"
		}
	}
	if "`show'"!="" {
		noi di "histogram!=blank, pctile!=blank passed (line 560)"
	}			

	**# standard
	if "`pctile'"=="" & "`histogram'"=="" & "`byest'"=="" & "`pwcorr'"=="" & `anzvar'>=2  {
		capture frame drop rdscfrnobs
		if _rc==0 {
			display as error "Frame rdscfrnobs found and dropped. Probably, nobsdes5 did not terminate correctly last time."
		}
		capture frame put `id' `varlist' `by' `bydupl' `touse', into(rdscfrnobs)	
		capture frame change rdscfrnobs
		qui keep if `touse'
		local intlist = ""
		local mislist = ""
		local lablist = ""
		local labmiss = "miss"
		local anzvar : word count `clist'
		local i 2
		local j 2
		local flag 0			
		
		/*  
		- sets value specified by miss() to missing 
		- generates dummy variable that is 1 if variable is missing
		- variable labels are getting lost by collapsing. Therefore local 
		  labextvar is created as list of variable labels where labextvar 
		  is set to miss for variables that have no variable label
		*/
		qui while `j' <= `anzvar' {
			local extvar_`i' : word `j' of `clist'	

			if "`bydupl'"=="" {	
				tempvar extvar_`i'_nvalid 
				summarize `extvar_`i''
				if "`miss'"!="" {
					replace `extvar_`i'' =. if `extvar_`i''==`miss'
				}		
				generate `extvar_`i'_nvalid' = (`extvar_`i''!=.)	
			}

			/* anders hochzählen extvar_`i' : word `j' of `clist', Dopplung */
			else if "`bydupl'"!="" {	
				tempvar extvar_`i'_nvalid ndupl		
				if "`extvar_`i''" != "(dupl)" {
					summarize `extvar_`i''
					if "`miss'"!="" {
						replace `extvar_`i'' =. if `extvar_`i''==`miss'
					}		
					generate `extvar_`i'_nvalid' = (`extvar_`i''!=.)	
				}
				else if "`extvar_`i''" == "(dupl)" {		
					local j = `j' + 1				
					local extvar_`i' : word `j' of `clist'	
					summarize `extvar_`i''
					if "`miss'"!="" {
						qui replace `extvar_`i'' =. if `extvar_`i''==`miss'
					}		
					generate `extvar_`i'_nvalid' = (`extvar_`i''!=.)	
					egen     `ndupl'      = total(`extvar_`i'_nvalid'), by(`bydupl')
					replace  `extvar_`i'' = `extvar_`i'' / `ndupl'  if `ndupl'!=0
					if "`show'"!="" {
						noi di "Step 3: `extvar_`i''    ndupl"
						noi table `bydupl', statistic(mean `ndupl') nototals
					}
				}
			}
			local labextvar: variable label `extvar_`i''
            local labextvar = subinstr("`labextvar'","(","",.)
            local labextvar = subinstr("`labextvar'",")","",.)
			
			
			local labextvar = "`labextvar' | "		
			local intlist `intlist' `extvar_`i''
			local lablist `lablist' `labextvar' 
			local vallist `vallist' `extvar_`i'_nvalid'	
			local i = `i' + 1
			local j = `j' + 1				
		}

		collapse (sum) `intlist' `vallist', by(`id' `by')
		mata condoutplab  = ("Variable",  "label")
		mata condoutp     = J(1,1,.)
		local flagno  = 0               /* no disclosure problem */
		local flagyes = 0               /*    disclosure problem */

		local anzvar : word count `intlist'	
		qui forvalues i = 1/`anzvar' {
			tempvar   id_tot minshare tot share r CR2 name crdum
			local     extvar        : word `i' of `intlist'
			local     extvar_nvalid : word `i' of `vallist'	
			
			/* Variable labels are reestablished and formatted */
			tokenize "`lablist'", parse("|")	
			local labextvar `1'
			if "`labextvar'" == "miss" {
				local labextvar = ""
			}
			if "`2'"=="|"  {
				macro shift 2
				local lablist `*'
			}
*			noi lab `labextvar', maxlet(40) 
			local maxlet = 40
			local blank = `maxlet' * " "
			if length("`labextvar'") > `maxlet' {	
				local labextvar = substr("`labextvar'",1,`maxlet') + " .."
			}
			else if length("`labextvar'") <= `maxlet' & length("`labextvar'")!=0 {
				local labextvar = substr("`blank'",1,`maxlet'-length("`labextvar'")) + "   `labextvar'"
			}
			else if length("`labextvar'")==0 {
				local labextvar = "`blank'"
			}					
			label variable `extvar' "`labextvar'"
			local blank = (12-length("`extvar'")) * " "

			/* -----------   Calculation of concentration ratio   ----------- */
			egen      `id_tot'   = total(`extvar')              if `extvar_nvalid'!=0, by(`id')  
			egen      `minshare' = min(`extvar'/`id_tot'*100)   if `extvar_nvalid'!=0, by(`by')
			egen      `tot'      = total(`extvar')              if `extvar_nvalid'!=0, by(`by') 
			generate  `share'    = `extvar'/`tot'*100           if `extvar_nvalid'!=0
			replace   `share'    = 0                            if `extvar_nvalid'==0 & `tot'==0 
 			if "`miss'"!="" {
				egen  `name'     = total(`extvar'!=. & `extvar'!=`miss'), by(`by') 	
			}
			else {
				egen  `name'     = total(`extvar_nvalid'!=0),		 	  by(`by')	
			}
			
			gsort `by' -`share'
			if "`by'"=="" {
				generate `r'   = sum(1)           	if `name'>0
			}
			else {
				by `by': generate `r' = sum(1)    	if `name'>0
			}
			egen     `CR2'		 = total(`share')	 if `r'<=2, by(`by')
			* max CR2 may be larger than 100 percent, therefore create crdum
			generate  `crdum'    = abs(`CR2') > 85 & abs(`CR2') <= 100
			qui summarize `crdum'
			if r(max)==1 {
			    qui summarize `CR2' if `crdum' == 1				
			    local kritcr   = abs(r(mean))				
			}
			else {
			    qui summarize `CR2'				
				local kritcr   = abs(r(max))
			}
			format %8.0f `CR2'								
			
			/* -------------------------------------------------------------- */
			if `share'[3]==`share'[2] {
				noisily display "Note: Second largest value occurs more than once"  
			}			

			replace   `name'  	 = . if `r'!=1	
			qui summarize `name'
			local kritnobs = r(min)					

			/* --------   condensed output in case of no by option  
			              or with by option but notable, preparation   ------ */

            if `kritnobs'>=`_minobs' & (`kritcr'<=85 | `kritcr'>100) {	
                local verweigert = "nein"
            }
			if `kritnobs'>=`_minobs' & (`kritcr'>85 & `kritcr'<=100) {	
                local verweigert = "Dominanz"
            }
            if (`kritnobs'<`_minobs' & `kritnobs'>0) & (`kritcr'<=85 | `kritcr'>100) {	
                local verweigert = "Fallzahl"
            }
			if (`kritnobs'<`_minobs' & `kritnobs'>0) & (`kritcr'>85 & `kritcr'<=100) {	
                local verweigert = "beides"
            }
						  
						  
			if "`by'"=="" {
				if "`verweigert'"=="nein" {
					local flagno = 1
					mata condoutp    = condoutp    \ `kritnobs'
					mata condoutplab = condoutplab \ ("`extvar' `blank' ", "- `labextvar'")		
				}
				else if "`verweigert'"=="Dominanz" {
					local flagyes = 1
					local crmsg`i'     "`extvar' `blank' - `labextvar'     Share of largest two IDs > 85% and <= 100%"						
				}	
				else if "`verweigert'"=="Fallzahl" {
					local flagyes = 1						
					local crmsg`i'     "`extvar' `blank' - `labextvar'     number of distinct IDs (`id') too small"
				}						
				else if "`verweigert'"=="beides" {
					local flagyes = 1						
					local crmsg`i'     "`extvar' `blank' - `labextvar'     Share of largest two IDs > 85% and number of distinct IDs (`id')    too small"
				}						
			}				

			if "`by'"!="" & "`notable'"!="" {
				if "`verweigert'"=="nein" {
					local flagno = 1
					mata condoutp    = condoutp    \ `kritnobs'
					mata condoutplab = condoutplab \ ("`extvar' `blank' - ", "`labextvar'")		
				}
				else if "`verweigert'"=="Dominanz" {
					local flagyes = 1
					local crmsg`i'     "`extvar' `blank' - `labextvar'     Share of largest two IDs > 85% and <= 100%"						
				}	
				else if "`verweigert'"=="Fallzahl" {
					local flagyes = 1						
					local crmsg`i'     "`extvar' `blank' - `labextvar'     number of distinct IDs (`id') too small"
				}						
				else if "`verweigert'"=="beides" {
					local flagyes = 1						
					local crmsg`i'     "`extvar' `blank' - `labextvar'     Share of largest two IDs in at least one cell > 85% and <= 100% and number of distinct IDs (`id')    too small"
				}						
			}				
				

		    /* ---  complete output in case of by option but not notable  --- */
						   
			if "`generate'" != "" {
				egen cr2_`id'_`extvar' = mean(`CR2'),  by(`by') 
			} 
			if "`show'"!="" {
				noi di "Step 3: `vs'; `extvar'    CR2"
				noi table `by', statistic(mean `CR2')
			}
			if `anzvar'>=1 & "`by'"!="" & "`notable'"=="" {
				display ""
				noi mess_by `id' `extvar' `name' `CR2', `notable' minobs(`_minobs') labelby(`labextvar') by(`by')		
			}
		}	

		
		/* -------------   condensed output in case of no by 
		                   option or with by option but notable   ----------- */
		noisily 
		if `anzvar'>=1 & "`by'"=="" {
			display as text  _newline "Output control"	
			if `flagno'==1 {				
				mata condoutp    = condoutp[2::rows(condoutp),.]
				mata condoutplab = condoutplab[2::rows(condoutplab),.]	
				mata s0          = ("",  "nobs")					
				display as text "No problem with dominance and sufficient number of distinct IDs (" as smcl "{it:`id'}" _asis ") (=nobs) of " _newline "variable:" 	
				mata _matrix_list(condoutp,condoutplab,s0)		
			}
			if `flagyes'==1 {
				display as error _newline "D I S C L O S U R E problem: " 
				forvalues i = 1/`anzvar' {
					if "`crmsg`i''" != "" {
						noi display as text "`crmsg`i''" 
					}
					else {
						display _continue
					}
				}
			}
		}
		else if `anzvar'>=1 & "`by'"!="" & "`notable'"!="" {
			display as text  _newline "Output control"	
			if `flagno'==1 {				
				mata condoutp    = condoutp[2::rows(condoutp),.]
				mata condoutplab = condoutplab[2::rows(condoutplab),.]	
				mata s0          = ("",  "nobs")					
				display as text "No problem with dominance and sufficient number of distinct IDs (" as smcl "{it:`id'}" _asis ") (=nobs) of " _newline "variable:" 	
				mata _matrix_list(condoutp,condoutplab,s0)		
			}
			if `flagyes'==1 {
				display as error _newline "D I S C L O S U R E problem: " 
				forvalues i = 1/`anzvar' {
					if "`crmsg`i''" != "" {
						noi display as text "`crmsg`i''" 
					}
					else {
						display _continue
					}
				}
			}
		}

		noi di _newline							
		if "`show'"!="" {
			noi di "Step 4: feed back to data"
		}
		qui capture frame change default
		qui if "`generate'" != "" {
			if "`by'"=="" {
				foreach var in `intlist' {	
					capture drop rdscfrnobs	
					cap drop tg_`id'_`var'
					if "`miss'"=="" {
					    egen byte  tg_`id'_`var' = tag(`id')   if `var'!=. & `touse'						
					}
					else if "`miss'"!="" {
					    egen byte  tg_`id'_`var' = tag(`id')   if `var'!=. & `touse' & `var'!=`miss'
					}
					frlink m:1 `id', frame(rdscfrnobs) 
					frget cr2_`id'_`var', from(rdscfrnobs)
					quietly replace cr2_`id'_`var' = .         if tg_`id'_`var'==0
				}
			}
			else if "`by'"!="" {
				foreach var in `intlist' {	
					capture drop rdscfrnobs						
					cap drop tg_`id'_`var'
					if "`miss'"=="" {
					    egen byte  tg_`id'_`var' = tag(`id' `by')   if `var'!=.	& `touse'
					}
					else if "`miss'"!="" {
					    egen byte  tg_`id'_`var' = tag(`id' `by')   if `var'!=. & `touse' & `var'!=`miss'
					}
					frlink m:1 `id' `by', frame(rdscfrnobs) 					
					frget cr2_`id'_`var', from(rdscfrnobs)
					quietly replace cr2_`id'_`var' = .              if tg_`id'_`var'==0
				}
			}
		}
		capture frame drop rdscfrnobs	
		capture drop rdscfrnobs
	}

	if "`show'"!="" {
		noi di "Step 6"
	}
	
	
	**# pwcorr
	if "`pwcorr'"!="" {
		local anzvar : word count `noidlist'
        qui {
        noisily display ""
        noisily display as text "Output control for variables  " as smcl "{it:`noidlist' }" _asis 
        tempvar s tg
        egen `tg' = tag(id)
        local flag = 0
        forvalues i = 1/`anzvar' {
	        local x : word `i' of `noidlist'	
            forvalues j = `i'/`anzvar' {
	            local y : word `j' of `noidlist'	
		        egen `s' = total((`x'!=.) * (`y'!=.)) if `touse', by(id)
		        count if `tg'==1 & `s'>0 &  `touse'
		        if r(N)<`_minobs' {
			        if `flag'==0 {
			        noi di in red "D I S C L O S U R E Problem: 				
				    local flag = 1
			    }
			    noi display as text "Number of distinct ids for `x' and `y':  " r(N)
		    }
		    drop `s' 
	    }
    }
    }
		
	}
	
	
	/* ------------------   identifier only, no variables   ----------------- */
	
	**# no variables
	local anzvar : word count `varlist'	
	if `anzvar'==1 {		
		if  "`by'"=="" & "`byest'"=="" {
			qui count if `name'==1
			local kritnobs = r(N)	
			if (`kritnobs'<`_minobs' & `kritnobs'>0) {	
				display as error "D I S C L O S U R E problem: "
				local rmin "too small"
			}
			else {
				local rmin = `kritnobs'
			}		
			display as text "Number of distinct values for identifier " as smcl "{it:`id'}"  _asis " :   `rmin'"
		}
		else if "`by'"!="" | "`byest'"!="" {
			local anzby : word count `by'
			if `anzby'==1 {	
				local labby : variable label `by'
				if length("`labby'") > 25 {	
					local labby = "[" + substr("`labby'",1,25) + " ..]"
				}
				else if length("`labby'") <= 25 & length("`labby'")!=0 {
					local labby = "[`labby']"
				}
			}
			tempvar hv
			qui egen      `hv' = total(`name') if `touse', by(`by')			
			qui summarize `hv'                 if `hv'!=0			
			local kritnobs = r(min)	
			display as text "Output control for variable " as smcl "{it:`by' `labby'}"	_asis
			if (`kritnobs'<`_minobs' & `kritnobs'>0) {
				display as error "D I S C L O S U R E problem: "				
				local rmin "too small"
			}
			else {
				local rmin = `kritnobs'
			}		
			if "`notable'"!="" {
				display as text "Smallest number of distinct values for identifier " as smcl "{it:`id'}" _asis " for " as smcl "{it:`by' `labby'}"	_asis " : 	`rmin'"
			}
			else if "`notable'"=="" {
				display as text "Number of distinct values for identifier " as smcl "{it:`id'}" _asis " for each " as smcl "{it:`by'}" _asis " :"	
				nobreak {
					sort   `by'
					rename `hv' nobs
					tempvar tg_id_by
					egen   `tg_id_by' = tag(`by') if nobs!=.
					list   `by' nobs              if `tg_id_by'==1, noobs sep(0)
					rename  nobs `hv'
				}
			}
		}
	}
	
	
	if "$endmark"!="" {
		local endtext "$endmark"
	}		
	if "`endmark'"!="" {
		local endtext = "`endmark'"
	}
	if "`endmark'"!="" | "$endmark"!="" {
		di "`endtext'"
	}
	
	if "`osor'" != "" {
		sort `osor'
	}
	capture label drop pgrforerr 
	version `vu'	
end		


/* ----------------   Report (message) in case of by option   --------------- */
capture program drop mess_by
program define mess_by, rclass
	syntax [varlist] [if] [in] [, BY(varlist) BYREGIO(varlist) NOTABle SHOW /*
	    */ MINOBS(int 5) LABELBY(string)]

	tokenize `varlist'
	local id        "`1'"
	local extvar    "`2'"
	local name      "`3'"
	local CR2       "`4'"
    local labextvar	"`labelby'"

	tempname touse
	mark `touse' `if' `in'

	tempvar crdum
	generate `crdum' = abs(`CR2') > 85 & abs(`CR2') <= 100
	qui summarize `crdum'
	if r(max)==1 {
	    qui summarize `CR2' if `crdum' == 1				
	    local kritcr   = abs(r(mean))				
	}
	else {
	    qui summarize `CR2'				
		local kritcr   = abs(r(max))
	}

	qui summarize `name'
	local kritnobs = r(min)	
	if (`kritnobs'<`minobs' & `kritnobs'>0) {
		local rmin "too small"
	}
	else {
		local rmin = `kritnobs'
	}

	display ""
	display as text "Output control for variable " as smcl "{it:`extvar' [`labelby']}"	
	if (`kritnobs'<`minobs' & `kritnobs'>0) | (`kritcr'>85 & `kritcr'<=100)  {	
		display as error "D I S C L O S U R E problem: "	
	}
	if `kritcr'>85 & `kritcr'<=100 & "`by'"=="" {
		display as text "Share of largest two IDs > 85% and <= 100%" 
	}				
	else if `kritcr'>85 & `kritcr'<=100 & "`by'"!="" {
		display as text "Share of largest two IDs in at least one cell > 85% and <= 100%" 
	}				
	else if `kritcr'<=85 | `kritcr'>100 {
		display as text "No problem with dominance " 
	}

	if "`by'"=="" {
		display as text "Number of distinct IDs (" as smcl "{it:`id'}" _asis ") of variable " as smcl "{it:`extvar'}" _asis " :   `rmin'"
	}
	else if "`by'"!="" {			
		local anzby : word count `by'
		if `anzby'==1 {	
			local labby : variable label `by'
			if length("`labby'") > 25 {	
				local labby = "[" + substr("`labby'",1,25) + " ..]"
			}
			else if length("`labby'") <= 25 & length("`labby'")!=0 {
				local labby = "[`labby']"
			}
		}
		
		if "`notable'"!="" {
		    display as text "Smallest number of distinct IDs (" as smcl "{it:`id'}" _asis ") of variable " as smcl "{it:`extvar'}" _asis " for  " as smcl "{it:`by' `labby'}" _asis " : `rmin'"
		}
		else if "`notable'"=="" {
			display as text "Number of distinct IDs (" as smcl "{it:`id'}" _asis ") of variable " as smcl "{it:`extvar'}" _asis " for each " as smcl "{it:`by'}" _asis " :"

			if (`kritcr'>85 & `kritcr'<=100)| (`kritnobs'>0 & `kritnobs'<`minobs') { 
				nobreak {
					rename `CR2' CR2 
					rename `name' nobs								
					sort   `by' 
					tempvar tg_id_by
					egen   `tg_id_by' = tag(`by') if nobs!=.
					list   `by' nobs CR2 if nobs!=. & `tg_id_by'==1, noobs sep(0)  abb(15)						
					rename  CR2 `CR2'
					rename nobs `name' 										
				}
			}
			if (`kritcr'<=85 |`kritcr'>100) & `kritnobs'>=`minobs' {
				nobreak {
					rename `name' nobs								
					sort   `by'
					tempvar tg_id_by
					egen   `tg_id_by' = tag(`by') if nobs!=.
					list   `by' nobs              if `tg_id_by'==1, noobs sep(0) abb(15)
					rename nobs `name' 					
				}
			}
		}
	}
end	

/* --------------   Report (message) in case of pctile option   ------------- */
capture program drop mess_pctile
program define mess_pctile, rclass
	syntax [varlist] [if] [in] [, BY(varlist) MINOBS(int 5) PCTILE(string) /*
	    */ LABELBY(string) PSTRING(string) NOTABle SHOW]

	tokenize `varlist'
	local id        "`1'"
	local extvar1   "`2'"
	local CR2	    "`3'"		
	local name      "`4'"	
	local pgr       "`5'"	
	local pgrhv     "`6'"	
	local extvar    "`7'"	
	local pstr = subinstr("`pstring'"," ", "", .)

	tempname touse pgrstr 
	mark `touse' `if' `in'

	tokenize "`pctile'"
	local pctilelist `*'	
	local i = 1
	foreach pct in `pctilelist' {
		if `pct'>0 & `pct'<100 {		
			local i = `i' + 1
		}
		else if `pct'<=0 {
			local i = `i' + 1				
		}			
	}
	local psep = `i'		

	qui {

    summarize `name'                          if `touse' & `name'==0
	list    if `touse' & `name'==0
	local min0 = r(N)
    summarize `name'                          if `touse' & `name'!=0
	local kritnobs = r(min)		
	if (r(min)>0 & r(min)<`minobs') {
		local rmin = "too small"
	}
	else {
		local rmin = r(min)
	}	
	generate  `pgrstr' = string(`pgrhv')  
	summarize `pgr'
	replace   `pgrstr' = "max" 	  if `pgr'==r(max)	
	
	tempvar crdum
	generate `crdum' = abs(`CR2') > 85 & abs(`CR2') <= 100
	qui summarize `crdum'
	if r(max)==1 {
	    qui summarize `CR2' if `crdum' == 1				
	    local kritcr   = abs(r(mean))				
	}
	else {
	    qui summarize `CR2'				
		local kritcr   = abs(r(max))
	}	
	if (`kritnobs'<`minobs' & `kritnobs'>0)|(`kritcr'>85 & `kritcr'<=100) {	
		local verweigert = "ja"
	}
	else if `kritnobs'>=`minobs' & (`kritcr'<=85 | `kritcr'>100) {	
		local verweigert = "nein"
	}
	
	
/*	summarize `CR2'                          if `touse'
	local kritcr  = r(max)
	local kritcr  = abs(`kritcr')	*/
	}	
	display ""
	display as text "Output control for variable " as smcl "{it:`extvar' [`labelby']}" _asis
	if "`verweigert'"=="ja" {
		noi display as error "D I S C L O S U R E problem: "
	}					
	
	if `kritcr'>85 & `kritcr'<=100 {
		noi display as text "Share of largest two IDs > 85% and <= 100%" 
	}				
	else if `kritcr'<=85 {
		noi display as text "No problem with dominance " 
	}

	if "`by'"=="" {
		if "`notable'"!="" {
			display as text "Smallest number of distinct IDs (" as smcl "{it:`id'}" ///
			_asis ") for a percentile (`pstr') of variable " as smcl "{it:`extvar'}" ///
			_asis " :  `rmin'"
		}
		else if "`notable'"=="" {
	        if "`verweigert'"=="ja" {
				sort    `pgr' `CR2' 
				display as text "Number of distinct IDs (" as smcl "{it:`id'}" ///
				_asis ") for each percentile (`pstring') of variable " ///
				as smcl "{it:`extvar'}" _asis " :"					
				nobreak {
					rename (`pgr' `CR2' `name') (pctile CR2 nobs)
					sort pctile
					if `min0'>0 {
					    list pctile nobs CR2 if `extvar'!=.               & pctile!=pctile[_n-1] & (CR2!=. | (CR2==. & nobs==0)), noobs abb(15) 
					}
					else if `min0'==0 {
					    list pctile nobs CR2 if `extvar'!=.               & pctile!=pctile[_n-1] & CR2!=., noobs	 abb(15) 
					}
					rename (pctile CR2 nobs) (`pgr' `CR2' `name')
					
				}
			}
			else if "`verweigert'"=="nein" {
				sort    `pgr' `CR2' 
				display as text "Number of distinct IDs (" as smcl "{it:`id'}" ///
				_asis ") for each percentile (`pstring') of variable " ///
				as smcl "{it:`extvar'}" _asis " :"	
				nobreak {
					rename  (`pgr' `pgrstr' `name')  (pctile value nobs)
					sort pctile	
					list pctile value nobs if ((nobs>`minobs' & nobs!=.)|nobs==0) & pctile!=pctile[_n-1] , noobs  abb(15)
					rename  (pctile value nobs)  (`pgr' `pgrstr' `name') 
				}
			}
		}
	}
	else if "`by'"!="" {
		local anzby : word count `by'
		if `anzby'==1 {	
			local labby : variable label `by'
			if length("`labby'") > 25 {	
				local labby = "[" + substr("`labby'",1,25) + " ..]"
			}
			else if length("`labby'") <= 25 & length("`labby'")!=0 {
				local labby = "[`labby']"
			}
		}
		if "`notable'"!="" {
			display as text "Number of distinct IDs (" as smcl "{it:`id'}" ///
			_asis ") for the smallest percentile (`pstr') of variable " ///
			as smcl "{it:`extvar'}" _asis " for " as smcl "{it:`by'}" ///
			_asis " :  `rmin'"
		}
		else if "`notable'"=="" {
	        if "`verweigert'"=="ja" {
				display as text "Number of distinct IDs (" as smcl "{it:`id'}" ///
				_asis ") for the smallest percentile (`pstr') of variable " ///
				as smcl "{it:`extvar'}" _asis " for " as smcl "{it:`by'}" ///
				_asis " :   `rmin'"
			}
			qui display "kritnobs  `kritnobs'    kritcr:  `kritcr'"
	        if "`verweigert'"=="ja" {
				nobreak {
					rename (`CR2' `name' `pgr') (CR2 nobs pctile)
					sort    `by' pctile	CR2
					if `min0'>0 {
					    list `by' pctile nobs CR2 if  pctile!=pctile[_n-1], sepby(`by') noobs  abb(15)												
					}
					else if `min0'==0 {
					    list `by' pctile nobs CR2 if `extvar'!=. & pctile!=pctile[_n-1] & CR2!=.,  sepby(`by') noobs abb(15) 
					}
					rename (CR2 nobs pctile) (`CR2' `name' `pgr')
				}
			}
			else if "`verweigert'"=="nein" {
				nobreak {
					rename  (`pgr' `pgrstr' `name')  (pctile value nobs)
					sort `by' pctile	
					list `by' pctile value nobs if nobs>`minobs' & nobs!=. & pctile!=pctile[_n-1], sepby(`by') noobs  abb(15)
					rename  (pctile value nobs)  (`pgr' `pgrstr' `name') 
				}
			}
		}
	}
end	

/* -------------------------   Formating labels   -------------------------- */
capture program drop lab
program define lab, rclass
	syntax [anything] [if] [in] [, MAXLET(int 25)]	
	noi di "`anything'"
	noi di "maxlet: " `maxlet'
	local blank = " "
	local blank = `maxlet' * "`blank'"
	if length("`anything'") > `maxlet' {	
		return local labextvar = substr("`anything'",1,`maxlet') + " .."
	}
	else if length("`anything'") <= `maxlet' & length("`anything'")!=0 {
		return local labextvar = substr("`blank'",1,`maxlet'-length("`anything'")) + "   `anything'"
	}
	else if length("`anything'")==0 {
		return local labextvar = "`blank'"
	}	
end

/* -----------------------   Calculates percentiles   ----------------------- */
capture program drop mypctile
program define mypctile , sortpreserve rclass byable(recall)

	syntax [varlist(default=none)] [if] [in] [, PCTILE(string) BY(varlist) BYDUPL(varlist)]
	
	tokenize `varlist'
	
	local id      "`1'"
	local extvar1 "`2'"
	local pgr     "`3'"
	local pgrstr  "`4'"
	
	tempname  touse 
	mark     `touse' `if' `in'	
	
	qui {
	* ergaenze Liste der Perzentile um das maximum (100)
	local anzpctile : word count `pctile'
	local pctile100 = "`pctile' 100"
	tokenize "`pctile100'"
	local pctilelist `*'
	local minpctile : word 1 of `pctilelist'
	local maxpctile : word `anzpctile' of `pctilelist'	
	local pstring = "min"

	cap label drop pgrforerr
	label define   pgrforerr 0 "default"	
		
	tempvar n nperc hv0 hv1 hv2 maxn xsm extvar_nvalid ndupl minnperc hvdum 
		
	if "`bydupl'"=="" {
		if "`by'"=="" {
			* Beispiel Fall mehrere Nullen für eine id: id 1 - 11 Nullen, 
			* ids 2 bis 10 - 1 Null, ids 1 bis 5 - 1 Wert größer Nullen
			* vergib Ränge:
			* sort var id rang: Bereich 1 -  1 id,  Bereich 2 - 10 ids
			* sort var rang id: Bereich 1 - 10 ids, Bereich 2 -  1 id
			* Sortierung des Forschers übernehmen, aber wie?
			sort `touse' `extvar1' `id', stable		
			generate  `n' = sum(1)	                 if `extvar1'!=. & `touse'
		}
		else if "`by'"!="" { 	
			sort   `by' `touse' `extvar1' `id', stable		
			bysort `by': generate  `n' = sum(1)      if `extvar1'!=. & `touse'
		}
	}
	else if "`bydupl'"!="" {
		generate `extvar_nvalid'  = (`extvar1'!=. & `touse')						
		egen     `ndupl'          = total(`extvar_nvalid'), by(`bydupl')
		if "`by'"=="" {		
		    sort   `extvar1' `id', stable				
		    generate `n' = sum(1/`ndupl')
		}
		else if "`by'"!="" { 			
		    sort   `by' `extvar1' `id', stable				
		    bysort `by': generate `n' = sum(1/`ndupl')
		}
		replace  `extvar1' = `extvar1'/`ndupl'		
	}
	egen	 `maxn'     = max(`n')       if `extvar1'!=., by(`by')   
	generate `nperc'    = `n' / `maxn'	    	
	egen	 `minnperc' = min(`nperc')   if `extvar1'!=., by(`by') 
	* xsm is (`extvar'[_n]+`extvar'[_n+1])/2 if n x pct is integer
	generate `xsm'		  = `extvar1'    	         if `extvar1'!=. & `touse'
	
	* W_(n-1)==P; since W_(n)>W_(n-1) n smallest n with W_(n)>P  
	* percentiles
	local i = 1
	foreach pct in `pctilelist' {
		replace `xsm' = (`xsm'[_n]+`xsm'[_n+1])/2 if abs(`nperc'-`pct'/100)<.0000001 & `n'!=`maxn'
		local i = `i' + 1
	}
	* categories
	local i = 1
	foreach pct in `pctilelist' {
        replace `pgr' = `i'                   if `pgr'==0 & `nperc'<=(`pct'/100 - .0000001)
		local i = `i' + 1
	}	
	** For maximum extvar pgr==0. Set pgr to largest category : i-1
	replace  `pgr' = `i'-1 				      if `pgr'==0 
	generate `hv0' = (`pgr'==`pgr'[_n-1]+1) 
	* `hv0'==1 & `n'!=`maxn': these are the precentiles; move one observation back
	replace `pgr'    = `pgr'-1                if `hv0'==1 
	*& `n'!=`maxn'    /* nummerisch */
    ** kleinstes Perzentil zunächst "nicht" besetzt, deshalb kleinster Wert
	replace `pgr'  = 1                        if `minnperc'>`minpctile'/100 & `nperc'==`minnperc'
	generate `hvdum' = (`minnperc'>`minpctile'/100 & `nperc'==`minnperc')  if `touse'
	summarize `hvdum' 
	local mindum = r(max)
	
	if "`bydupl'"!="" {	
		egen    `hv2'    = min(`pgr'), by(`id' `xsm')
		replace `pgr'    = `hv2' 
	}	
    * value labels
	local i = 1
	foreach pct in `pctilelist' {
		label define pgrforerr `i' "p`pct'", add
		local i = `i' + 1
	}	
	local i    = `i'-1
	label define pgrforerr `i' "max", modify
	sum `pgr' if `touse' 
	if `anzpctile' == r(max) {
	    local i    = `i'-1		
	    label define pgrforerr `i' "max", modify		
	}
	if `mindum' == 1 {
	    label define pgrforerr 1 "min", modify				
	}
	
    * text 	
	local i = 1
	foreach pct in `pctilelist' {
		local pstring = "`pstring'"+"<=p`pct'" 
		local i = `i' + 1
	}	

	* maximum may be equal for consecutive `pgr's  ->  `pgrstr' is shorter then
	egen    `hv1'    = max(`xsm'), by(`by' `pgr') 
	replace `pgrstr' = `hv1'                         /* percentiles */
	label values `pgr' pgrforerr	
	return local pstring = subinstr("`pstring'","p100","max",1)
	}
end	
