{smcl}

{title:Title}

{phang}
{bf:nobsdes5} {hline 2} Disclosure control for tables, graphs and aggregation 
according to the rules of the Bundesbank RDSC (at least 5 observations)

{marker syntax}{...}
{title:Syntax}

{p 8 17 2}
{cmdab:nobsdes5}
{it:identifier } [({it:dupl})] {it:variable}
{ifin}
{opt , by(varlist)}
{opt byregio(varlist)}
{opt notab}
{opt generate}
{opt name(varname)}
{opt replace}
{opt miss(string)}
{opt pctile(string)}
{opt histogram(string)}
{opt bydupl(varlist)}

{opt end(string)}

{marker description}{...}
{title:Description}

{pstd}
{cmd:nobsdes5} replaces {cmd:nobsdes} which is not valid anymore since disclosure
 control rules have changed.
{smcl}
{pstd}
The command {cmd:nobsdes5} followed by the name of an identifier and the name of
 one or more variables reports the number of distinct values for the identifier 
 for non-missing values of the variables. It is a shortcut for {bf:nobsdes5 identifier if variable!=. & identifier!=""}
 (or {identifier!=.}). With {bf:by} it does the same showing a table unless 
 {bf:notab} is specified. If {bf:generate} is specified, a new variable 
 {bf:tg_identifier_variablename} is created. It is 1 for each identifier in each 
 group defined by the levels of the {bf:by} variable and otherwise zero. An own 
 variable name can be provided by the {bf:name} option. 
{smcl}
{pstd}
At the same time {bf:nobsdes5} calculates whether the share of the two largest 
identifiers in the sum of the variables (their absolutes) (concentration ratio 2)
is higher than 85% meaning these two being dominant. {bf:generate} creates 
{bf:cr2_identifier_variablename}. This name cannot be user specified.
{smcl}
{pstd}
The command is useful for exploring data where the records refer to more than 
one level, for example longitudinal data where each record refers to foreign 
direct investments by domestic firms.
{smcl}
{pstd}
If data is aggregated for further use, it is also useful to document that the 
rules of the Bundesbank RDSC are not violated.
{smcl}
{pstd}
It is possible to specify only the identifier. Then the number of distinct 
non-missing values for the identifier is reported.
{smcl}

{title:Options for use with {cmd:nobsdes5}}

{phang}
{cmd:by(}{it:varlist}{cmd:)} provides the names of one or more deterministic 
 categorical variable(s). If one of these variables denotes region, you have to 
 use the {bf:byregio} option. Do not use the {bf:by} option and the {bf:byregio}
 option together.

{phang}
{cmd:byregio(}{it:varlist}{cmd:)} is {error: obligatory} in case of publication
 of results on regional level because requirements are stricter.

{phang}
{cmd:bydupl(}{it:varlist}{cmd:)} weights the variable following (dupl) with the 
 inverse frequency of the observations specified by the varlist of bydupl() so 
 that the weighted sum equals the repeated value. Can be used for example if a
 table shall be published where each value is counted once.
 
{phang}
{cmd:notab} supresses the table of the {bf:by} option, of the {bf:byest} option,
 and of the {bf:percentile} option. It is rarely used with the {bf:by} option 
 because in general you have to display the table. Use {bf:notab} only with very
 large tables. 

{phang}
{cmd:name(}{it:varname}{cmd:)} supplies the name of the new variable which 
 contains the number of distinct values of the varlist by levels defined by the 
 categorical variable supplied in the {bf:by}.

{phang}
{cmd:generate} does the same as the {bf:name(}{it:varname}{cmd:)} option but 
 uses as name the variable name with tg_ and cr2_ in front.

{phang}
{cmd:replace} overwrites the variable specified by {bf:name(}{it:varname}{cmd:)}.

{phang}
{cmd:miss()} allows the specification of a value treated as (additional) missing
 e.g. 0 and is {error: obligatory} for continuous variables with the exception of 
 percentiles (and histograms).

{phang}
{cmd:pctile()} allows the specification of percentiles. Ascending order is 
 required. {cmd:nobsdes5} counts the number of unique identifiers and verifies the 
 dominance criterion. In case of several variables and different percentiles "|" has 
 to be used as delimiter. If only one set of percentiles is specified this is
 applied for each variable.  
 {p 8 8}
 If {bf:miss(0)} is specified, quantiles are calculated taken zeros into account 
 nonetheless. However, the number of distinct entities are determined ignoring  
 zeros. If zeros should not be taken into account for calculating the quantiles,
 you have to use {it:if varname!=0}.  
{smcl}
{phang}
{cmd:histogram()} enumerates the distinct identifiers in each bin of a histogram.
 Implies {opt notab}. The number of bins has to be specified in brackets. In case 
 of several variables a specific number of bins for each variable can be specified. 
 If only one number of bins is specified this applies for each histogram.

{phang}
{cmd:end()} specifies an end mark to the nobsdes5 output. The file nobsclean.do 
 (available from the RDSC homepage) then erases the text between 
 {it:nobsdes5 ... endmark}. It is also possible to specify the global macro 
 global endmark {it:"endmark"} to erase all {it:nobs .... endmark}  output.


{marker reserved}{...}
{title:Reserved words}

{bf:rdscfrnobs} name of frame and variable
{bf:pgrforerr} name of label


{marker examples}{...}
{title:Examples}

The following examples show the use of the syntax. They are not necessarily the  
way output control should be done. For the latter, see "Further examples".

1) Consider a longitudinal data set in which each record corresponds to a foreign 
direct investment (id_fil) by a domestic firm (id). The subject identity is in the 
variable id, and the direct investment is identified by the variable id_fil, within id. 
{com}
clear
set obs 100
generate id     = _n if _n<=10
replace  id     = 11 if _n> 10
generate id_fil = _n * 10

{txt}For the number of different foreign direct investments use
{com}nobsdes5 id_fil 

{txt}For the number of different domestic firms with foreign direct investments calculate
{com}egen tg_id_id_fil = tag(id id_fil)
{com}egen anz_id_fil_id = total(tg_id_id_fil), by(id)
{com}label variable anz_id_fil_id "repeated values"
{com}list , sep(0)
{txt}Then use
{com}nobsdes5 id (dupl) anz_id_fil_id, bydupl(id)

{com}. nobsdes5 id (dupl) anz_id_fil_id, bydupl(id)
{txt}Note: Second largest value occurs more than once

Output control

{error:D I S C L O S U R E problem:}
anz_id_fil_id   -                             repeated values     Share of largest two IDs > 85% and <= 100%


2) Create a panel with random data:
{com}
clear
set obs 100
generate id = int(_n/10)+1
generate time = _n-10*(id-1)+1
generate x = runiform()
generate dum = id==1
replace x = . if id==11
{txt}
2a) Output control for the mean or the total of x
{com}nobsdes5 id x
{txt}
{com}. nobsdes5 id x
{txt}
Output control
No problem with dominance and sufficient number of distinct IDs (id) (=nobs) of 
variable:
                                                          nobs
x             -                                             10

{txt}
2b) Output control for a dummy:
{com}nobsdes5 id, by(dum)

{com}. nobsdes5 id, by(dum)
{txt}{error:Warning} Is id an identifier?
Output control for variable dum 
{error:D I S C L O S U R E problem:}
Number of distinct values for identifier id for each dum :

  +------------+
  | dum   nobs |
  |------------|
  |   0     10 |
  |   1      1 |
  +------------+

{txt}

3) Graph showing the mean of x (save the graph as .png-file)
   (taken from nobsdes5xmpl.do, see below):

{com}nobsdes5 id x, by(year land) 
{com}egen meanx = mean(x) if year!=2002, by(year land)
{com}egen tgland = tag(year land)
{com}gsort -tgland year land
{com}line meanx year if land=="BE" & tgland
{com}* graph save graphname.png
{txt}


4) Table with a variable with replicated values:

{com}sysuse auto, clear
{com}egen income = total(price), by (foreign)
{com}label variable income "income"
{com}table foreign, statistic(mean income) nototals

{com}nobsdes5 make (dupl) income mpg, by(foreign) bydupl(make foreign)
{txt}
{com}. table foreign, statistic(mean income) nototals
{txt}
--------------------
           |    Mean
-----------+--------
Car origin |        
  Domestic |  315766
  Foreign  |  140463
--------------------

{com}. 
{com}. nobsdes5 make (dupl) income mpg, by(foreign) bydupl(make foreign)
{txt}
Output control for variable income [income]
No problem with dominance 
Number of distinct IDs (make) of variable income for each foreign :

  +-----------------+
  |  foreign   nobs |
  |-----------------|
  | Domestic     52 |
  |  Foreign     22 |
  +-----------------+

Output control for variable mpg [Mileage (mpg)]
No problem with dominance 
Number of distinct IDs (make) of variable mpg for each foreign :

  +-----------------+
  |  foreign   nobs |
  |-----------------|
  | Domestic     52 |
  |  Foreign     22 |
  +-----------------+



{marker author}{...}
{title:Author}
		Harald Stahl
		Harald.Stahl@bundesbank.de
        
{marker alsosee}{...}
{title:Also see}
nobsreg5, maxrdsc

Further examples: 
R:\Zentrale\ZB-S\Daten_FDSZ_Gastforscher\1_gafo\examples_rdsc\nobsdes5xmpl.do
