/* nobsreg5.ado -- hst 05.01.2021 /29.01.2021 */

*! Analyses regression output with respect to data disclosure problems.
*! version 1.0.2  13.01.2022 
*!  bug with if ... fixed
*!  comment on brackets added
*!  accelerated by using preserve restore 
*! version 1.0.1  11.03.2021 
*!  ivreghdfe added
*! version 1.0.0  29.01.2021 
*!  new disclosure rules: at least 5 observations and 20 with regional information

capture program drop nobsreg5
capture program drop interact2
capture program drop contvar2
capture program drop diskvar12
capture program drop diskvar22

program define nobsreg5
	
	syntax [varlist(default=none)] [if] [in] [, regio(varname) ENDmark(string)]

	local anzvar : word count `varlist'	
	
	if `anzvar'==0 & "$idlist"!="" { 
		local idl = "$idlist"
	}
	else if `anzvar'>=1 { 
		local idl = "`varlist'"
	}
	else if "`_dta[Ridlist]'"!="" {
		local idl : char _dta[Ridlist]	
	}
	else {
		di as error "You have to specify a variable with the IDs of the entity to be protected!"
		di as error "Disclosure control aborted"
	}
	local befehl = `"`e(cmdline)'"'
	if strpos(`"`befehl'"', " if ") > 0 {
		local befehl = substr(`"`befehl'"',1,strpos(`"`befehl'"', " if ")) 
	}
	tokenize `"`befehl'"', parse(",")	
	tokenize `1'
	if "`1'"=="bootstrap" {
		di ""
		di as text "For bootstrap, it is better to run an artificial OLS regression with the same variables."
		tokenize `"`e(cmdline)'"', parse(":")	
		di `"`3'"'
		tokenize `"`3'"', parse(",")
		tokenize `"`1'"'
		macro shift
	}
	else {
		macro shift                             /* command       */
		macro shift                             /* LHS-variable  */
	}
	local reglist `*'							/* RHS-variables */

	local stop  : word count `reglist'
	local anzid : word count `idl'

	macro shift
	local i = 0
	di "" 
	foreach x in `idl' {
		if "`e(cmd)'"=="gsem" {
			di ""
			di as error "Sorry, nobsreg does not work with gsem!"
			di as text  "Please run an artificial OLS regression with the same variables."
			continue, break
		}
		if "`e(cmd)'"=="ivregress" | "`e(cmd)'"=="ivreg2" | "`e(cmd)'"=="ivreg28" | "`e(cmd)'"=="ivreg29" | "`e(cmd)'"=="ivreg210" | "`e(cmd)'"=="ivreghdfe" {
			tokenize "`e(cmdline)'", parse("=")		
			tokenize `"`1'"', parse("(")
			local x3 "`3'"
			tokenize `"`1'"', parse(" ")
			macro shift
			macro shift
			macro shift
			local reglist `*' `x3'				/* RHS-variables */				
		}

		if "`e(cmd)'"=="heckman" {
			tokenize "`e(cmdline)'", parse(",")	
			local x3 "`3'"
			tokenize `"`1'"', parse(" ")
			macro shift
			local reglist1 `*' 				/* RHS-variables */				
			tokenize `"`x3'"', parse("=")	
			tokenize `"`3'"', parse(")")
			tokenize `"`1'"', parse(" ")				
			local reglist2 `*' 				/* RHS-variables */				
			local reglist `reglist1' `reglist2'
		}			
		tempvar name eins anz
	    capture confirm str variable `x'
		if _rc==0 {
			quietly egen byte `name' = tag(`x') if e(sample)==1 & `x'!="" 	
		}
		else if _rc!=0 {
			quietly egen byte `name' = tag(`x') if e(sample)==1 & `x'!=. 	
		}
		quietly count if `name'==1 
		local tot = string(r(N))
		local i = `i'+1
		if r(N)<5 & r(N)>0 {
			di as error "D I S C L O S U R E problem: Number of distinct values for variable   `x'   too small"
		}
		else {
			di as text "Number of distinct values for variable  `x'  :   " r(N) 			
		}
		local f1 = 0
		mata: nobsmat`i'  = J(0,1,.)
		mata: nobsmats`i' = ("Variable", "distinct IDs")\("--------", "------------")
		
		local reglist2 = ""
		local reglist3 = ""
		local lagflag = 0
		di ""
		foreach xx in `reglist' {
			if (("`xx'"=="if") | ("`xx'"==",")) {				
				continue, break
			}
			if substr("`xx'",1,1)=="[" {
				display as error ""Output control interrupted." as text " Error in `xx' : " as error "[" as text " and " as error "]" as text " not allowed."
				display ""				
				continue, break
			}			
			if strpos("`xx'","(")!=0 | strpos("`xx'",")")!=0 {
				display as error "Output control interrupted." as text " Error in `xx' : " as error "(" as text " and " as error ")" as text " - brackets in general - not allowed."
				display ""
				continue, break				
			}	
			
			if strpos("`xx'","*")!=0 {
				qui des `xx', varlist
				local wcstar = r(varlist)
				local reglist2 `reglist2' `wcstar'
			}	
			else if strpos("`xx'","?")!=0 {
				qui des `xx', varlist
				local wcquest = r(varlist)
				local reglist2 `reglist2' `wcquest'
			}				
			else if strpos("`xx'","-")!=0 {
				qui des `xx', varlist
				local wcdash = r(varlist)
				local reglist2 `reglist2' `wcdash'
			}	
			else {
				local reglist2 `reglist2' `xx'			
			}
		}
		
		foreach xx in `reglist2' {			
			local hvle   = length("`xx'")
			local dotpos = strpos("`xx'",".")
			local lt     = upper(substr("`xx'",1,1))
			local lt2    = substr("`xx'",1,2)
			if `dotpos'!=0 & "`lt2'"=="L("{
				local xxld = substr("`xx'",1,`dotpos'-1)
				local hvle = length("`xx'")
				local l2var = substr("`xx'",`dotpos'+1,`hvle'-`dotpos')	
				if strpos("`xxld'","(")!=0 & strpos("`xxld'","/")!=0 & strpos("`xxld'",")")!=0 {
					local firstlag = substr("`xxld'",3,strpos("`xxld'","/")-3)
					local lastlag  = substr("`xxld'",strpos("`xxld'","/")+1,`dotpos'-strpos("`xxld'","/")-2)
					local laglist  = ""
					forvalues k=`firstlag'/`lastlag' {
						local newlag = "L`k'.`l2var'"
						local laglist `laglist' `newlag'
					}
				local reglist3 `reglist3' `laglist'	
				}
            }
			else if `dotpos'==0 & "`lt2'"=="L("{
				local lagflag = 1
				local newlag  = substr("`xx'",3,length("`xx'"))					
				local laglist = `newlag'				
			}
			else if `dotpos'!=0 & substr("`xx'",`dotpos'-1,2)==")." {
				local hvle    = length("`xx'")
				local l2var   = substr("`xx'",`dotpos'+1,`hvle'-`dotpos')	
				local newlag  = substr("`xx'",1,`dotpos'-2)
				local lagflag = 0
				local laglist `laglist' `newlag'
				local laglist2 = ""					
				foreach k of numlist `laglist' {
					local newlag2 = "L`k'.`l2var'"
					local laglist2 `laglist2' `newlag2'
				}
				local reglist3 `reglist3' `laglist2'
			}		
			else if `lagflag'==1 {				
				local laglist `laglist' `xx'
			}				
			else {
				local reglist3 `reglist3' `xx'	
				local lt  ""
				local lt2 ""
			}
		}

		local reglist4 = ""			/* Interaktionen */			
		foreach xx in `reglist3' {
			local spos1  = strpos("`xx'","#")     /* position of first  # */
			/* Interaktion des Typs ##; hierbei muss eine diskrete Variable den Vorsatz i. bekommen */
			if strpos("`xx'","##")!=0 {	
				interact2 "`xx'"
				local ia = r(varname)
				local reglist4 `reglist4' `ia'
			}
			else {
				local reglist4 `reglist4' `xx'
			}
		}			

		local textincmd = "incmdline" 
		foreach xx in `reglist4' {
			if (("`xx'"=="if") | ("`xx'"==",") | (substr("`xx'",1,1)=="[")) {
				continue, break
			}
			local dotpos = strpos("`xx'",".")
			local spos1  = strpos("`xx'","#")     /* position of first  # */

			/* Interaktion (#) */
			if `spos1'!=0 {
				tokenize "`xx'", p(#)
				local intactlist `*'
				local xxdata = ""
				local xxname = ""
				local lt     = ""
				foreach var in `intactlist' {
					if "`var'"!="#" {
						local dp = strpos("`var'",".")
						capture local z = real(substr("`var'",1, `dp'-1))
						if substr("`var'",1,2)=="i." |(substr("`var'",1,2)=="ib")|`dp'==0 {
							local stripvar  = substr("`var'",`dp'+1,length("`var'")-`dp')
							local stripname = "`stripvar'"                 /* header */
							local lt        = "`lt' I"	
						}
						else if (substr("`var'",1,3)=="i1."|substr("`var'",1,3)=="i2."|substr("`var'",1,3)=="i3."|substr("`var'",1,3)=="i4.") {
							di "S o r r y, do not understand  `var'   !!!" 
							continue, break
						}
						* interaction with continuous variable: identified, if
						* continuous variable for category k available for ID j 
						* only -> create xxdata
						else if substr("`var'",1,2)=="c."|substr("`var'",1,2)=="l." {
							local stripvar  = substr("`var'",`dp'+1,length("`var'")-`dp')
							local stripname = "`stripvar'"
							local hv        = upper(substr("`var'",1,1))
							local lt        = "`lt' `hv'"	
						}
						* z.B. 400.land
						else if _rc==0 {
							tempvar helpvar
							generate `helpvar' =  `var'
							local stripvar  = "`helpvar'"
							local stripname = "`var'"
							local lt        = "`lt' "+  substr("`var'",1, `dp'-1)
						}						
						
						else {
							di "S o r r y, do not understand  `var'   !!!" 
							continue, break
						}
						if "`xxdata'"=="`regiovar'" | "`stripvar'"=="`regiovar'"{
							local regio = 1
						}
						else {
							local regio = 0				
						}				
						local xxdata = "`xxdata' `stripvar'"
						local xxname = "`xxname'#`stripname'"	
					}
				}
				local xxdata = trim("`xxdata'")
				local xxname = substr("`xxname'",2,length("`xxname'"))
				local lt     = trim("`lt'")		
				diskvar22 `x' `xxdata' if e(sample)==1, prefix(`lt') textincmd(`textincmd') name("`xxname'")
				local textincmd = r(txincmd)
				mata: nobsmats`i' = (nobsmats`i'\("`xxname'", ""))
				mata: nobsmats`i' = (nobsmats`i'\Xi12mat)	
			}
			if `dotpos'!=0 {
				local lt  = upper(substr("`xx'",1,1))
				local lt2 = substr("`xx'",1,2)
			}
			else if `dotpos'==0 {
				local lt  = ""
				local lt2 = ""
			}

			/* Stetige Variablen und Rest */
			if `spos1'==0 & inlist("`lt'","L","C","F","D","S") & "`lt2'"!="L(" & ~("`lt'">="0" & "`lt'"<="9") {
				local xxname = substr("`xx'",`dotpos'+1,length("`xx'")-`dotpos')	
				contvar2 `x' `xxname' if e(sample)==1, prefix(`lt2') textincmd(`textincmd')
				local textincmd = r(txincmd)
				mata: nobsmats`i' = (nobsmats`i'\("`xx'", "")\Xtmat)						
			}
			else if `spos1'==0 & "`lt2'"!="i." & "`lt2'"!="ib" & substr("`xx'",1,3)!="." & "`lt2'"!="L(" & ~("`lt'">="0" & "`lt'"<="9") {			
				local xxname = substr("`xx'",`dotpos'+1,length("`xx'")-`dotpos')	
				contvar2 `x' `xxname' if e(sample)==1 , prefix(`lt2') textincmd(`textincmd')
				local textincmd = r(txincmd)
				mata: nobsmats`i' = (nobsmats`i'\("`xx'", "")\Xtmat)						
			}

			/* Diskrete Variable */
			if `spos1'==0 & "`lt'"=="I" {
				local xxdata = substr("`xx'",`dotpos'+1,length("`xx'")-`dotpos')
			}
			else if	`spos1'==0 & ("`lt'">="0" & "`lt'"<="9") {
				local xxdata = "`xx'"
			}
			if `spos1'==0 & ("`lt'"=="I" | ("`lt'">="0" & "`lt'"<="9")) {			
				if "`xxdata'"=="`regiovar'" {
					local regio = 1
				}
				else {
					local regio = 0				
				}
				diskvar12 `x' `xxdata' if e(sample)==1, prefix(`lt') textincmd(`textincmd') minobs(`regio')
				local textincmd = r(txincmd)
				mata: nobsmats`i' = (nobsmats`i'\("`xxdata'", "")\Ximat)						
			}
		}

		if `anzid'!=1 {
			if `i'==1 {
				mata: nobsmats`i' = nobsmats`i'\("--------","------------")\("`x'", "`tot'")\("","")
			}
			if `i'==`anzid' {
				mata: nobsmats`i' = nobsmats`i'\("--------","------------")\ ("","")\("`x'", "`tot'")
				mata: nobsmats1 = (nobsmats1,nobsmats`i'[.,2])
				mata: nobsmats1[rows(nobsmats1),1] = nobsmats`i'[rows(nobsmats`i'),1]
			}
		}
		if `anzid'==1 {
			mata: nobsmats1 = nobsmats1\("--------","------------")\("`x'", "`tot'")
		}
		mata: nobstable = nobsmats1
	}

	if "$endmark"!="" {
		local endtext "$endmark"
	}		
	if "`endmark'"!="" {
		local endtext = "`endmark'"
	}
	if "`endmark'"!="" | "$endmark"!="" {
		di "`endtext'"
	}
end	




program define interact2, rclass

	local xx `1'
	local laenge = length("`xx'")         /* stringlength         */
	local dotpos = strpos("`xx'",".")     /* position of dot      */
	local spos1  = strpos("`xx'","#")     /* position of first  # */
	local spos2  = 0                      /* position of second # */

	/* Schreibe Text:							 

	Interaktionen können von links nach rechts gelöst werden, d. h.

	1. a##b##c = (a##b)##c = (a a#b b)##c 
		
	= (a a#b b)  (a a#b b)#c   c
	=  a a#b b  a#c a#b#c b#c  c

		
	2. a##b#c = (a##b)#c = (a a#b b)#c 	=  a#c a#b#c b#c 
		
	3. a#b##c = (a#b)##c = a#b c  a#b#c
		

	Definiere zunächst die Klammer  */
	
	local klammer    = ""

	/* ************************************************************** */		
	/*                     Abfrage, ob # oder ##                      */
	/* ************************************************************** */			
	if substr("`xx'",`spos1'+1,1)=="#" {
		local spos2 = `spos1'+2                /* ##  found */
		local einfach1 = 2				
	}
	else {
		local spos2 = `spos1'+1                /* #   found */
		local einfach1 = 1
	}

	/* ************************************************************** */
	/*                       Erste Komponente                         */	
	/* ************************************************************** */
	local anfang = substr("`xx'",1,`spos1'-1)      /* varname up to # */
	local dp     = strpos("`anfang'",".")

	* kategorial
	capture local z = real(substr("`xx'",1, `dp'-1))
	if substr("`xx'",1,2)=="i."|(substr("`xx'",1,2)=="ib") {	
		local xx1  = "i."+substr("`anfang'",`dp'+1,length("`anfang'")-`dp')
		local klammer `klammer' `xx1'	
	}
	else if `dp'==0 {
		local xx1 = "i."+"`anfang'"
		local klammer `klammer' `xx1'
	}	
	else if `dp'!=0  & _rc==0 {
		local xx1 = "`anfang'"
		local klammer `klammer' `xx1'
	}		
	* stetig
	else if substr("`anfang'",1,2)=="c."|substr("`anfang'",1,2)=="l." {
		local xx1 = "`anfang'"
		local klammer `klammer' `xx1'
	}
	* anderes
	else {
		di "S o r r y, do not understand  `anfang'   !!!" 
		continue, break
	}
	local i = 1

	/* ************************************************************** */
	/*                      Weitere Komponenten                       */
	/* ************************************************************** */			
	local rest   = substr("`xx'",`spos2',`laenge'-`spos2'+1)  
	local laenge = length("`rest'")
	local i = `i'+1
	local j = `i'

	while "`rest'" != "" {
		/* Abfrage, ob # oder ## */
		local spos3  = strpos("`rest'","#")     /* position of first  # */
		if `spos3'!=0 {
			if substr("`rest'",`spos3'+1,1)=="#" {
				local spos4 = `spos3'+2                /* ##  found */
				local einfach2 = 2
			}
			else {
				local spos4 = `spos3'+1                /* #   found */			
				local einfach2 = 1
			}
			* extrahiere Komponente und gib Rest weiter 
			local hv   = substr("`rest'",1,`spos3'-1)  
			local rest = substr("`rest'",`spos4',`laenge')  
			local i = `i'+1
		}
		else if `spos3'==0 {
			* extrahiere Komponente und setze Rest auf "" 	
			local hv   = substr("`rest'",1,length("`rest'")) 
			local dp   = strpos("`hv'",".")
			if `dp'==0 {
				local hv = "i."+substr("`rest'",1,length("`rest'"))
			}
			local rest = ""					
		}
		if `j'==3 {
			local rest = ""	
			tokenize "`klammer'", parse("")				
			if `einfach1'==1 & `einfach2'==2 {
				local klammer `2' `hv' `2'#`hv'
			}
			else if `einfach2'==1 & `einfach1'==2 {
				local klammer `1'#`hv' `2'#`hv' `3'#`hv' 
			}
			else if `einfach2'==2 & `einfach1'==2 {
				local klammer `klammer' `1'#`hv' `2'#`hv' `3'#`hv' `hv'
			}
		}  
		else if `j'<3 {
			local dp = strpos("`hv'",".")
			* kategorial 
			capture local z = real(substr("`hv'",1, `dp'-1))
			if substr("`hv'",1,2)=="i."| _rc==0 |(substr("`hv'",1,2)=="ib")|`dp'==0 {			
*				local xx2 = "i."+substr("`hv'",`dp'+1,length("`hv'")-`dp')
				local xx2 = "`hv'"
				local klammer `klammer' `xx1'#`xx2' `xx2'	
			}
			* stetig
			else if substr("`hv'",1,2)=="c."|substr("`hv'",1,2)=="l." {
				local xx2 = substr("`hv'",1,length("`hv'"))
				local klammer `klammer' `xx1'#`xx2' `xx2'			
				local hv2 `xx2'
			}
			* anderes
			else {
				di "S o r r y, do not understand  `hv'   !!!" 
				continue, break
			}
		}
		local j = `j'+1
*		di "i2: `klammer'"
		return local varname `klammer'
	}
end	


/* Diskrete Variable */
* if `dotpos'!=0 & "`lt'"=="I" & `spos2'==0 & `f1'==0 & `fall'==0 {
* diskvar xx x if e(sample)==1, prefix(I) fdp(1)

program define contvar2, rclass

	syntax [varlist] [if] [in] [, PREfix(string) TEXTincmd(string)]

	* `1' ist eine ID aus idlist
	local x  "`2'"

	qui {
	if "`prefix'"=="c." {	
		local prefix = "c"
	}
	if "`prefix'"=="l." {	
		local prefix = "l"
	}

	preserve
	tempname touse 
	mark `touse' `if' `in'

	tempvar id ivar tvar anz eins tgivar anz1 sanz 
	generate `id' = `1'	
	capture confirm str variable `id'
	if _rc==0 {
		replace  `touse' =  0 if `touse'==1 & `id'==""
	}
	else if _rc!=0 {
		replace  `touse' =  0 if `touse'==1 & `id'==.	
	}	
	drop if `touse'==0

	generate byte `tvar' = (`x'!=0 & `x'!=.)	

	egen byte `eins'   = tag(`id' `tvar')	
	keep if `eins'==1
	egen      `anz'    = total(`eins')     		if `tvar'==1 
	summarize `anz'                         
	local ramin = r(min)
	replace       `anz'    = 0 					if `anz'==.					
	egen     byte `tgivar' = tag(`tvar')		if `tvar'!=. & `anz'!=0 
	generate str  `sanz'   = string(`anz')	
	generate str  `ivar'   = string(`tvar')	
	replace       `ivar'   = "==0"				if `ivar'=="0"
	replace       `ivar'   = "<>0" 				if `ivar'=="1"
	mata: Xtmat = st_sdata(.,("`ivar'", "`sanz'"),"`tgivar'")	
	}
	if `ramin'>0 & `ramin'<5  {	
		if "`textincmd'"=="incmdline" {
			di "In " `"`e(cmdline)'"'
			return local txincmd = "notext"			
		}
		if "`prefix'"!="" {
			di as error "Potential D I S C L O S U R E problem: Number of distinct IDs (`1') for variable  `prefix'.`x'  too small"
			di as text "(in case of a continuous variable x: 1 means x!=0 & x!=.  and  0 means x==0)			
		}
		else if "`prefix'"=="" {
			di as error "Potential D I S C L O S U R E problem: Number of distinct IDs (`1') for variable  `x'  too small"
			di as text "(in case of a continuous variable x: 1 means x!=0 & x!=.  and  0 means x==0)
		}
	}	
	else {
		return local txincmd = "`textincmd'"
	}
	restore
end


program define diskvar12, rclass
		
	syntax [varlist (fv)] [if] [in] [, PREfix(string) TEXTincmd(string) MINOBS(int 0)]

	* `1' ist eine ID aus idlist
	local x  "`2'"

	qui {
	tempname touse 
	mark `touse' `if' `in'

	/* Jede Kombination aus ID und Ausprägung von ivar wird nur einmal markiert */
	tempvar ivar anz eins tgivar sanz
	local id "`1'"
	tokenize "`2'", parse(",")
	local x  "`1'"	

	preserve
	capture confirm str variable `id'
	if _rc==0 {
		replace  `touse' = 0 					if `touse'==1 & `id'==""
	}
	else if _rc!=0 {
		replace  `touse' = 0 					if `touse'==1 & `id'==.	
	}	
	drop if `touse'==0

	generate  str  `ivar' = string(`x')	
	egen byte `eins' = tag(`id' `ivar')		
	keep if `eins'==1
	egen      `anz'  = total(`eins')			if `ivar'!="", by(`ivar')
	summarize `anz'                             
	local ramin = r(min)	
	egen     byte `tgivar' = tag(`ivar')		if `ivar'!=""
	generate str  `sanz'   = string(`anz')						
	mata: Ximat = st_sdata(.,("`ivar'", "`sanz'"),"`tgivar'")
	}

	local hihi = 5*(1-`minobs')+20*`minobs'
	if `ramin'>0 & `ramin'<5*(1-`minobs')+20*`minobs' {
		if "`textincmd'"=="incmdline" {
			di "In " `"`e(cmdline)'"' 
			return local txincmd = "notext"						
		}
		di as error "Potential D I S C L O S U R E problem: Too few observations in at least one category"
		di as text " "
		di %15s "`x'" 
		list `ivar' if `tgivar' & `anz'>=1 & `anz'<5*(1-`minobs')+20*`minobs', noheader nolabel noobs sep(0)						
		di ""
	}
	else {
		return local txincmd = "`textincmd'"
	}
	restore
end


program define diskvar22, rclass

	syntax [varlist] [if] [in] [, PREfix(string) TEXTincmd(string) MINOBS(int 0) NAME(string)]
*	set trace on
	local anzvar : word count `varlist'		
	local anzfix : word count `prefix'		

	qui {
	tokenize "`prefix'"
	forvalues i = 1(1)`anzfix' {
		local p     = `i'+1
		local lt`p' = "``i''"
	}
	tokenize "`varlist'"

	preserve
	tempname touse 
	mark `touse' `if' `in'
	
	tempvar ivar ivar1 tvar anz anz1 eins tgivar tgivar1 sanz 
	local id "`1'"				               /* ID */
	capture confirm str variable `id'	
	if _rc==0 {
		replace  `touse' =  0 if `touse'==1 & `id'==""
	}
	else if _rc!=0 {
		replace  `touse' =  0 if `touse'==1 & `id'==.	
	}	
    drop if `touse'==0
	
	if "`lt2'"!="C"&"`lt2'"!="L" {
		gen str `ivar'  = string(`2') 	
		gen str `ivar1' = `ivar'
		replace `ivar1' = "."                if `ivar'=="0"
	}
	else if "`lt2'"=="C"|"`lt2'"=="L" {
		gen str `ivar'  = string(`2'!=0 & `2'!=.)			
	}

/*	gen  str  `ivar'   = string(`2')      Strings wegen Verknüpfung der Daten */
	forvalues i = 3(1)`anzvar' {
		if "`lt`i''"!="C" & "`lt`i''"!="L" {
			generate str `tvar' = string(``i'') 
		}
		else if "`lt`i''"=="C"|"`lt`i''"=="L" {
			generate  str `tvar'   = string(``i''!=0 & ``i''!=.)				
		} 

		replace `ivar'  = `ivar'+"."+`tvar'   
		capture  drop `tvar'
	}

	/* Jede Kombination aus ID und Ausprägung von ivar wird nur einmal markiert */

	* i.xx1#i.xx2
	egen byte `eins'    = tag(`id' `ivar') if e(sample)==1
	keep if `eins'==1
	egen      `anz'     = total(`eins')    if e(sample)==1 & `ivar'!="", by(`ivar')
	summarize `anz'
	local ramin = r(min)	
	egen byte `tgivar'  = tag(`ivar')      if e(sample)==1 & `ivar'!=""
	gen  str  `sanz'    = string(`anz')    
	mata: Xi12mat = st_sdata(.,("`ivar'", "`sanz'"),"`tgivar'")
*	mata: Zi12mat = st_data(.,"`anz'","`tgivar'")					
	}
	if `ramin'>0 & `ramin'<5*(1-`minobs')+20*`minobs' {
		if "`textincmd'"=="incmdline" {
			display "In " `"`e(cmdline)'"' 
			return local txincmd = "notext"			
		}
		di as error "Potential D I S C L O S U R E problem: Too few observations in at least one category"
		di as text "(in case of a continuous variable x: 1 means x!=0 & x!=.  and  0 means x==0)
		di " "
		di %15s "`name'" 
		list `ivar' if `tgivar' & `anz'>=1 & `anz'<5*(1-`minobs')+20*`minobs', noheader sep(0) nolabel noobs
		di ""
	}
	else {
		return local txincmd = "`textincmd'"
	}
*	set trace off
	restore
end


/* textincmd : textincmd=="incmdline"	=> 		di "In `e(cmdline)'"  
			   textincmd=="notext" 		=>	    di nothing                    */
