{smcl}

{title:Title}

{phang}
{bf:nobsreg5} {hline 2} Disclosure control for regressions according to the rules of the Bundesbank RDSC (at least 5 observations)

{marker syntax}{...}
{title:Syntax}

{p 8 17 2}
{cmdab:global idlist}
[{varlist}]

{p 8 17 2}
{cmdab:nobsreg5}
{ifin}
{opt , regio(varname) end(string)}

{p 8 17 2}
or

{p 8 17 2}
{cmdab:nobsreg5}
[{id}]
{ifin}
{opt , regio(varname) end(string)}

{marker description}{...}
{title:Description}

{pstd}
{cmd:nobsreg5} replaces {cmd:nobsreg} which is not valid anymore since disclosure control rules have changed.
{smcl}
{pstd}
{cmd:nobsreg5} following a regression displays the number of distinct values
for the identifier(s). Additionally it analyzes whether some of the RHS-variables 
refer to less than five distinct values of the identifiers and whether the RHS 
of the regression includes one or more dichotomous variables where one or both 
values occur less often than five times in the regression. The analysis rests 
upon the command and the variables provided by {bf:e(cmdline)} and the sample 
provided by {bf:e(sample)} that are left from the previous regression. Therefore,
in case the regression is restricted by an {it:if}, this {it:if} does not have 
to be repeated.
{pstd}
Depending on the interpretation, there may be a confidentiality problem. If no 
such potential problem can be identified, the only output of nobsreg5 is the number 
of distinct values of the identifier(s).
{smcl}
{pstd}
In case of several regressions with the same list of identifiers, it is convenient 
to define the global macro {bf:idlist} for this list just once and to use 
{bf:nobsreg5} after the regressions without varlist. {bf:nobsreg5} can also be 
used by writing the list of identifiers immediately after {bf:nobsreg5}. Then 
there is no need for the global $idlist. 
{smcl}
{pstd}
The variable list of the regression must not contain brackets ( ).  
{smcl}
{pstd}
{bf:nobsreg5} creates a mata matrix {bf:nobstable} that displays for each variable the 
number of distinct ids as a spin-off. This may be useful if e.g. data mining is 
a concern. It does not have to be provided for disclosure control.
{smcl}


{title:Options for use with {cmd:nobsreg5}}


{phang}
{cmd:regio(}{it:varname}{cmd:)} supplies the name of the variable, which provides
 the regional data and is {error: obligatory} in case of publication of results on regional
 level because requirements are stricter. 

{phang}
{cmd:end()} specifies an end mark to the nobsreg5 output. The file nobsclean.do (available from the RDSC homepage) then
erases the text between {it:nobsreg5 ... endmark}. It is also possible to specify the global macro global endmark {it:"endmark"} 
to erase all {it:nobs .... endmark}  output.

{marker examples}{...}
{title:Examples}

Random data:

A do-file with the code:

{txt}
{com}clear
{com}set obs 100
{com}gen id = int(_n/10)+1
{com}gen time = _n-10*(id-1)+1
{com}gen y = runiform()
{com}gen x = runiform()
{com}gen z = runiform()
{com}gen dum1 = _n==5|_n==57
{com}gen dum2 = _n==5|_n==57|_n==78
{com}gen dum3 = id==1
{com}replace x = . if id==11
{com}replace z = 0 if id!=1
{txt}
{com}global idlist "id"
{com}reg y x z dum1 dum2 dum3
{com}nobsreg5
{com}mata: nobstable

displays:

{com}. reg y x z dum1 dum2 dum3
{txt}
{txt}      Source {c |}       SS           df       MS      Number of obs   ={res}        99
{txt}{hline 13}{c +}{hline 34}   F(5, 93)        = {res}     2.03
{txt}       Model {c |} {res} .928489288         5  .185697858   {txt}Prob > F        ={res}    0.0810
{txt}    Residual {c |} {res} 8.49095951        93   .09130064   {txt}R-squared       ={res}    0.0986
{txt}{hline 13}{c +}{hline 34}   Adj R-squared   ={res}    0.0501
{txt}       Total {c |} {res}  9.4194488        98  .096116825   {txt}Root MSE        =   {res} .30216

{txt}{hline 13}{c TT}{hline 11}{hline 11}{hline 9}{hline 8}{hline 13}{hline 12}
{col 1}           y{col 14}{c |}      Coef.{col 26}   Std. Err.{col 38}      t{col 46}   P>|t|{col 54}     [95% Con{col 67}f. Interval]
{hline 13}{c +}{hline 11}{hline 11}{hline 9}{hline 8}{hline 13}{hline 12}
{space 11}x {c |}{col 14}{res}{space 2} .0702151{col 26}{space 2} .1122476{col 37}{space 1}    0.63{col 46}{space 3}0.533{col 54}{space 4}-.1526864{col 67}{space 3} .2931166
{txt}{space 11}z {c |}{col 14}{res}{space 2}-.3375057{col 26}{space 2} .4738481{col 37}{space 1}   -0.71{col 46}{space 3}0.478{col 54}{space 4}-1.278474{col 67}{space 3} .6034627
{txt}{space 8}dum1 {c |}{col 14}{res}{space 2} .8894591{col 26}{space 2} .3742136{col 37}{space 1}    2.38{col 46}{space 3}0.020{col 54}{space 4}  .146345{col 67}{space 3} 1.632573
{txt}{space 8}dum2 {c |}{col 14}{res}{space 2}-.4669244{col 26}{space 2} .3074818{col 37}{space 1}   -1.52{col 46}{space 3}0.132{col 54}{space 4}-1.077522{col 67}{space 3} .1436736
{txt}{space 8}dum3 {c |}{col 14}{res}{space 2}-.0180345{col 26}{space 2} .3392333{col 37}{space 1}   -0.05{col 46}{space 3}0.958{col 54}{space 4}-.6916847{col 67}{space 3} .6556156
{txt}{space 7}_cons {c |}{col 14}{res}{space 2} .4816199{col 26}{space 2} .0630061{col 37}{space 1}    7.64{col 46}{space 3}0.000{col 54}{space 4} .3565023{col 67}{space 3} .6067374
{txt}{hline 13}{c BT}{hline 11}{hline 11}{hline 9}{hline 8}{hline 13}{hline 12}
{res}{txt}
{com}. nobsreg5

Number of distinct values for variable id :   10

{res}In regress y x z dum1 dum2 dum3
potential D I S C L O S U R E problem: Number of distinct ids (id) for variable z :   too small
In regress y x z dum1 dum2 dum3
potential D I S C L O S U R E problem: Number of distinct ids (id) for variable dum1 :   too small
In regress y x z dum1 dum2 dum3
potential D I S C L O S U R E problem: Number of distinct ids (id) for variable dum3 :   too small
{txt}
{com}. mata: nobstable
{res}        {txt}           1              2
     {c TLC}{hline 31}{c TRC}
   1 {c |}  {res}    Variable   distinct ids{txt}  {c |}
   2 {c |}  {res}    --------   ------------{txt}  {c |}
   3 {c |}  {res}           x               {txt}  {c |}
   4 {c |}  {res}         <>0             10{txt}  {c |}
   5 {c |}  {res}           z               {txt}  {c |}
   6 {c |}  {res}         <>0              1{txt}  {c |}
   7 {c |}  {res}        dum1               {txt}  {c |}
   8 {c |}  {res}         <>0              2{txt}  {c |}
   9 {c |}  {res}        dum2               {txt}  {c |}
  10 {c |}  {res}         <>0              3{txt}  {c |}
  11 {c |}  {res}        dum3               {txt}  {c |}
  12 {c |}  {res}         <>0              1{txt}  {c |}
  13 {c |}  {res}    --------   ------------{txt}  {c |}
  14 {c |}  {res}          id             10{txt}  {c |}
     {c BLC}{hline 31}{c BRC}


{marker author}{...}
{title:Author}

		Harald Stahl
		Harald.Stahl@bundesbank.de

        
{marker alsosee}{...}
{title:Also see}
nobsdes5, maxrdsc

Further examples:
R:\Zentrale\ZB-S\Daten_FDSZ_Gastforscher\1_gafo\examples_rdsc\nobsreg5xmpl.do
